@@ -31,7 +31,7 @@
 #include "configuration.h"
 #include "fileio.h"
 
-static inline int fileio_open_local(fileio_t *fileio)
+static inline int fileio_open_local(struct fileio *fileio)
 {
 	char access[4];
 
@@ -98,7 +98,7 @@ static inline int fileio_open_local(fileio_t *fileio)
 	return ERROR_OK;
 }
 
-int fileio_open(fileio_t *fileio, const char *url, enum fileio_access access,	enum fileio_type type)
+int fileio_open(struct fileio *fileio, const char *url, enum fileio_access access,	enum fileio_type type)
 {
 	int retval = ERROR_OK;
 
@@ -111,7 +111,7 @@ int fileio_open(fileio_t *fileio, const char *url, enum fileio_access access,	en
 	return retval;
 }
 
-static inline int fileio_close_local(fileio_t *fileio)
+static inline int fileio_close_local(struct fileio *fileio)
 {
 	int retval;
 	if ((retval = fclose(fileio->file)) != 0)
@@ -131,19 +131,19 @@ static inline int fileio_close_local(fileio_t *fileio)
 	return ERROR_OK;
 }
 
-int fileio_close(fileio_t *fileio)
+int fileio_close(struct fileio *fileio)
 {
 	int retval;
 
 	retval = fileio_close_local(fileio);
 
-	free(fileio->url);
+	free((void*)fileio->url);
 	fileio->url = NULL;
 
 	return retval;
 }
 
-int fileio_seek(fileio_t *fileio, uint32_t position)
+int fileio_seek(struct fileio *fileio, size_t position)
 {
 	int retval;
 	if ((retval = fseek(fileio->file, position, SEEK_SET)) != 0)
@@ -155,32 +155,34 @@ int fileio_seek(fileio_t *fileio, uint32_t position)
 	return ERROR_OK;
 }
 
-static inline int fileio_local_read(fileio_t *fileio, uint32_t size, uint8_t *buffer, uint32_t *size_read)
+static int fileio_local_read(struct fileio *fileio,
+		size_t size, void *buffer, size_t *size_read)
 {
-	*size_read = fread(buffer, 1, size, fileio->file);
-
-	return ERROR_OK;
+	ssize_t retval = fread(buffer, 1, size, fileio->file);
+	*size_read = (retval >= 0) ? retval : 0;
+	return (retval < 0) ? retval : ERROR_OK;
 }
 
-int fileio_read(fileio_t *fileio, uint32_t size, uint8_t *buffer, uint32_t *size_read)
+int fileio_read(struct fileio *fileio, size_t size, void *buffer,
+		size_t *size_read)
 {
 	return fileio_local_read(fileio, size, buffer, size_read);
 }
 
-int fileio_read_u32(fileio_t *fileio, uint32_t *data)
+int fileio_read_u32(struct fileio *fileio, uint32_t *data)
 {
 	uint8_t buf[4];
-	uint32_t size_read;
-	int retval;
-
-	if ((retval = fileio_local_read(fileio, 4, buf, &size_read)) != ERROR_OK)
-		return retval;
-	*data = be_to_h_u32(buf);
-
-	return ERROR_OK;
+	size_t size_read;
+	int retval = fileio_local_read(fileio, sizeof(uint32_t), buf, &size_read);
+	if (ERROR_OK == retval && sizeof(uint32_t) != size_read)
+		retval = -EIO;
+	if (ERROR_OK == retval)
+		*data = be_to_h_u32(buf);
+	return retval;
 }
 
-static inline int fileio_local_fgets(fileio_t *fileio, uint32_t size, char *buffer)
+static int fileio_local_fgets(struct fileio *fileio,
+		size_t size, void *buffer)
 {
 	if (fgets(buffer, size, fileio->file) == NULL)
 		return ERROR_FILEIO_OPERATION_FAILED;
@@ -188,40 +190,37 @@ static inline int fileio_local_fgets(fileio_t *fileio, uint32_t size, char *buff
 	return ERROR_OK;
 }
 
-int fileio_fgets(fileio_t *fileio, uint32_t size, char *buffer)
+int fileio_fgets(struct fileio *fileio, size_t size, void *buffer)
 {
 	return fileio_local_fgets(fileio, size, buffer);
 }
 
-static inline int fileio_local_write(fileio_t *fileio, uint32_t size, const uint8_t *buffer, uint32_t *size_written)
+static int fileio_local_write(struct fileio *fileio,
+		size_t size, const void *buffer, size_t *size_written)
 {
-	*size_written = fwrite(buffer, 1, size, fileio->file);
-
-	return ERROR_OK;
+	ssize_t retval = fwrite(buffer, 1, size, fileio->file);
+	*size_written = (retval >= 0) ? retval : 0;
+	return (retval < 0) ? retval : ERROR_OK;
 }
 
-int fileio_write(fileio_t *fileio, uint32_t size, const uint8_t *buffer, uint32_t *size_written)
+int fileio_write(struct fileio *fileio,
+		size_t size, const void *buffer, size_t *size_written)
 {
-	int retval;
-
-	retval = fileio_local_write(fileio, size, buffer, size_written);
-
+	int retval = fileio_local_write(fileio, size, buffer, size_written);
 	if (retval == ERROR_OK)
 		fileio->size += *size_written;
-
-	return retval;;
+	return retval;
 }
 
-int fileio_write_u32(fileio_t *fileio, uint32_t data)
+int fileio_write_u32(struct fileio *fileio, uint32_t data)
 {
 	uint8_t buf[4];
-	uint32_t size_written;
-	int retval;
-
 	h_u32_to_be(buf, data);
 
-	if ((retval = fileio_local_write(fileio, 4, buf, &size_written)) != ERROR_OK)
-		return retval;
+	size_t size_written;
+	int retval = fileio_write(fileio, 4, buf, &size_written);
+	if (ERROR_OK == retval && size_written != sizeof(uint32_t))
+		retval = -EIO;
 
-	return ERROR_OK;
+	return retval;
 }
