@@ -26,7 +26,7 @@
 #ifndef FILEIO_H
 #define FILEIO_H
 
-#include "types.h"
+#include <helper/types.h>
 
 #define FILEIO_MAX_ERROR_STRING		(128)
 
@@ -38,6 +38,7 @@ enum fileio_type
 
 enum fileio_access
 {
+	FILEIO_NONE,		/* open without any access (invalid mode) */
 	FILEIO_READ,		/* open for reading, position at beginning */
 	FILEIO_WRITE,		/* open for writing, position at beginning */
 	FILEIO_READWRITE,	/* open for writing, position at beginning, allow reading */
@@ -45,23 +46,28 @@ enum fileio_access
 	FILEIO_APPENDREAD,	/* open for writing, position at end, allow reading */
 };
 
-typedef struct fileio_s
-{
-	char *url;
-	long long size;
+struct fileio {
+	const char *url;
+	ssize_t size;
 	enum fileio_type type;
 	enum fileio_access access;
 	FILE *file;
-} fileio_t;
+};
 
-extern int fileio_write(fileio_t *fileio, uint32_t size, const uint8_t *buffer, uint32_t *size_written);
-extern int fileio_read(fileio_t *fileio, uint32_t size, uint8_t *buffer, uint32_t *size_read);
-extern int fileio_fgets(fileio_t *fileio, uint32_t size, char *buffer);
-extern int fileio_seek(fileio_t *fileio, uint32_t position);
-extern int fileio_close(fileio_t *fileio);
-extern int fileio_open(fileio_t *fileio, const char *url, enum fileio_access access, enum fileio_type type);
-extern int fileio_read_u32(fileio_t *fileio, uint32_t *data);
-extern int fileio_write_u32(fileio_t *fileio, uint32_t data);
+int fileio_open(struct fileio *fileio,
+	const char *url, enum fileio_access access, enum fileio_type type);
+int fileio_close(struct fileio *fileio);
+
+int fileio_seek(struct fileio *fileio, size_t position);
+int fileio_fgets(struct fileio *fileio, size_t size, void *buffer);
+
+int fileio_read(struct fileio *fileio,
+		size_t size, void *buffer, size_t *size_read);
+int fileio_write(struct fileio *fileio,
+		size_t size, const void *buffer, size_t *size_written);
+
+int fileio_read_u32(struct fileio *fileio, uint32_t *data);
+int fileio_write_u32(struct fileio *fileio, uint32_t data);
 
 #define ERROR_FILEIO_LOCATION_UNKNOWN	(-1200)
 #define ERROR_FILEIO_NOT_FOUND			(-1201)
