@@ -38,9 +38,6 @@
 #ifdef HAVE_NET_IF_H
 #include <net/if.h>
 #endif
-//#ifdef HAVE_NETINET_TCP_H
-//#include <netinet/tcp.h>
-//#endif
 #ifdef HAVE_SYS_IOCTL_H
 #include <sys/ioctl.h>
 #endif
@@ -57,24 +54,6 @@
 #endif
 
 
-int handle_rm_command(struct command_context_s *cmd_ctx, char *cmd,
-		char **args, int argc)
-{
-	if (argc != 1)
-	{
-		command_print(cmd_ctx, "rm <filename>");
-		return ERROR_INVALID_ARGUMENTS;
-	}
-
-	if (unlink(args[0]) != 0)
-	{
-		command_print(cmd_ctx, "failed: %d", errno);
-	}
-
-	return ERROR_OK;
-}
-
-
 /* loads a file and returns a pointer to it in memory. The file contains
  * a 0 byte(sentinel) after len bytes - the length of the file. */
 int loadFile(const char *fileName, void **data, size_t *len)
@@ -134,14 +113,11 @@ int loadFile(const char *fileName, void **data, size_t *len)
 	return ERROR_OK;
 }
 
-
-
-int handle_cat_command(struct command_context_s *cmd_ctx, char *cmd,
-		char **args, int argc)
+COMMAND_HANDLER(handle_cat_command)
 {
-	if (argc != 1)
+	if (CMD_ARGC != 1)
 	{
-		command_print(cmd_ctx, "cat <filename>");
+		command_print(CMD_CTX, "cat <filename>");
 		return ERROR_INVALID_ARGUMENTS;
 	}
 
@@ -149,45 +125,44 @@ int handle_cat_command(struct command_context_s *cmd_ctx, char *cmd,
 	void *data;
 	size_t len;
 
-	int retval = loadFile(args[0], &data, &len);
+	int retval = loadFile(CMD_ARGV[0], &data, &len);
 	if (retval == ERROR_OK)
 	{
-		command_print(cmd_ctx, "%s", (char *)data);
+		command_print(CMD_CTX, "%s", (char *)data);
 		free(data);
 	}
 	else
 	{
-		command_print(cmd_ctx, "%s not found %d", args[0], retval);
+		command_print(CMD_CTX, "%s not found %d", CMD_ARGV[0], retval);
 	}
 
 	return ERROR_OK;
 }
-int handle_trunc_command(struct command_context_s *cmd_ctx, char *cmd,
-		char **args, int argc)
+
+COMMAND_HANDLER(handle_trunc_command)
 {
-	if (argc != 1)
+	if (CMD_ARGC != 1)
 	{
-		command_print(cmd_ctx, "trunc <filename>");
+		command_print(CMD_CTX, "trunc <filename>");
 		return ERROR_INVALID_ARGUMENTS;
 	}
 
 	FILE *config_file = NULL;
-	config_file = fopen(args[0], "w");
+	config_file = fopen(CMD_ARGV[0], "w");
 	if (config_file != NULL)
 		fclose(config_file);
 
 	return ERROR_OK;
 }
 
-
-int handle_meminfo_command(struct command_context_s *cmd_ctx, char *cmd, char **args, int argc)
+COMMAND_HANDLER(handle_meminfo_command)
 {
 	static int prev = 0;
 	struct mallinfo info;
 
-	if (argc != 0)
+	if (CMD_ARGC != 0)
 	{
-		command_print(cmd_ctx, "meminfo");
+		command_print(CMD_CTX, "meminfo");
 		return ERROR_INVALID_ARGUMENTS;
 	}
 
@@ -195,48 +170,48 @@ int handle_meminfo_command(struct command_context_s *cmd_ctx, char *cmd, char **
 
 	if (prev > 0)
 	{
-		command_print(cmd_ctx, "Diff:            %d", prev - info.fordblks);
+		command_print(CMD_CTX, "Diff:            %d", prev - info.fordblks);
 	}
 	prev = info.fordblks;
 
-	command_print(cmd_ctx, "Available ram:   %d", info.fordblks);
+	command_print(CMD_CTX, "Available ram:   %d", info.fordblks);
 
 	return ERROR_OK;
 }
 
 
-int handle_append_command(struct command_context_s *cmd_ctx, char *cmd,
-		char **args, int argc)
+COMMAND_HANDLER(handle_append_command)
 {
-	if (argc < 1)
+	if (CMD_ARGC < 1)
 	{
-		command_print(cmd_ctx,
+		command_print(CMD_CTX,
 				"append <filename> [<string1>, [<string2>, ...]]");
 		return ERROR_INVALID_ARGUMENTS;
 	}
 
 	int retval = ERROR_FAIL;
 	FILE *config_file = NULL;
-	config_file = fopen(args[0], "a");
+
+	config_file = fopen(CMD_ARGV[0], "a");
 	if (config_file != NULL)
 	{
-		int i;
 		fseek(config_file, 0, SEEK_END);
 
-		for (i = 1; i < argc; i++)
+		unsigned i;
+		for (i = 1; i < CMD_ARGC; i++)
 		{
-			if (fwrite(args[i], 1, strlen(args[i]), config_file) != strlen(args[i]))
+			if (fwrite(CMD_ARGV[i], 1, strlen(CMD_ARGV[i]),
+					config_file) != strlen(CMD_ARGV[i]))
 				break;
-			if (i != argc - 1)
+			if (i != CMD_ARGC - 1)
 			{
 				if (fwrite(" ", 1, 1, config_file) != 1)
 					break;
 			}
 		}
-		if ((i == argc) && (fwrite("\n", 1, 1, config_file) == 1))
-		{
+		if ((i == CMD_ARGC) && (fwrite("\n", 1, 1, config_file) == 1))
 			retval = ERROR_OK;
-		}
+
 		fclose(config_file);
 	}
 
@@ -245,9 +220,9 @@ int handle_append_command(struct command_context_s *cmd_ctx, char *cmd,
 
 
 
-int handle_cp_command(struct command_context_s *cmd_ctx, char *cmd, char **args, int argc)
+COMMAND_HANDLER(handle_cp_command)
 {
-	if (argc != 2)
+	if (CMD_ARGC != 2)
 	{
 		return ERROR_INVALID_ARGUMENTS;
 	}
@@ -256,11 +231,11 @@ int handle_cp_command(struct command_context_s *cmd_ctx, char *cmd, char **args,
 	void *data;
 	size_t len;
 
-	int retval = loadFile(args[0], &data, &len);
+	int retval = loadFile(CMD_ARGV[0], &data, &len);
 	if (retval != ERROR_OK)
 		return retval;
 
-	FILE *f = fopen(args[1], "wb");
+	FILE *f = fopen(CMD_ARGV[1], "wb");
 	if (f == NULL)
 		retval = ERROR_INVALID_ARGUMENTS;
 
@@ -282,7 +257,7 @@ int handle_cp_command(struct command_context_s *cmd_ctx, char *cmd, char **args,
 			break;
 		}
 
-		command_print(cmd_ctx, "%zu", len - pos);
+		command_print(CMD_CTX, "%zu", len - pos);
 
 		pos += chunk;
 
@@ -292,10 +267,10 @@ int handle_cp_command(struct command_context_s *cmd_ctx, char *cmd, char **args,
 
 	if (retval == ERROR_OK)
 	{
-		command_print(cmd_ctx, "Copied %s to %s", args[0], args[1]);
+		command_print(CMD_CTX, "Copied %s to %s", CMD_ARGV[0], CMD_ARGV[1]);
 	} else
 	{
-		command_print(cmd_ctx, "Failed: %d", retval);
+		command_print(CMD_CTX, "Failed: %d", retval);
 	}
 
 	if (data != NULL)
@@ -304,7 +279,7 @@ int handle_cp_command(struct command_context_s *cmd_ctx, char *cmd, char **args,
 		fclose(f);
 
 	if (retval != ERROR_OK)
-		unlink(args[1]);
+		unlink(CMD_ARGV[1]);
 
 	return retval;
 }
@@ -427,25 +402,18 @@ void copydir(char *name, char *destdir)
 
 
 
-static int
-zylinjtag_Jim_Command_rm(Jim_Interp *interp,
-                                   int argc,
-		Jim_Obj * const *argv)
+COMMAND_HANDLER(handle_rm_command)
 {
-	int del;
-	if (argc != 2)
-	{
-		Jim_WrongNumArgs(interp, 1, argv, "rm ?dirorfile?");
-		return JIM_ERR;
-	}
+	if (CMD_ARGC != 1)
+		return ERROR_INVALID_ARGUMENTS;
 
-	del = 0;
-	if (unlink(Jim_GetString(argv[1], NULL)) == 0)
-		del = 1;
-	if (rmdir(Jim_GetString(argv[1], NULL)) == 0)
-		del = 1;
+	bool del = false;
+	if (rmdir(CMD_ARGV[0]) == 0)
+		del = true;
+	else if (unlink(CMD_ARGV[0]) == 0)
+		del = true;
 
-	return del ? JIM_OK : JIM_ERR;
+	return del ? ERROR_OK : ERROR_FAIL;
 }
 
 
@@ -649,41 +617,96 @@ static int zylinjtag_Jim_Command_mac(Jim_Interp *interp, int argc,
 
 }
 
+static const struct command_registration ioutil_command_handlers[] = {
+	{
+		.name = "cat",
+		.handler = handle_cat_command,
+		.mode = COMMAND_ANY,
+		.help = "display text file content",
+		.usage= "file_name",
+	},
+	{
+		.name = "trunc",
+		.handler = handle_trunc_command,
+		.mode = COMMAND_ANY,
+		.help = "truncate a file to zero length",
+		.usage= "file_name",
+	},
+	{
+		.name = "cp",
+		.handler = handle_cp_command,
+		.mode = COMMAND_ANY,
+		.help = "copy a file",
+		.usage = "src_file_name dst_file_name",
+	},
+	{
+		.name = "append_file",
+		.handler = handle_append_command,
+		.mode = COMMAND_ANY,
+		.help = "append a variable number of strings to a file",
+		.usage= "file_name [string ...]",
+	},
+	{
+		.name = "meminfo",
+		.handler = handle_meminfo_command,
+		.mode = COMMAND_ANY,
+		.help = "display free heap space",
+	},
+	{
+		.name = "rm",
+		.mode = COMMAND_ANY,
+		.handler = handle_rm_command,
+		.help = "remove a directory or file",
+		.usage = "file_name",
+	},
 
+	/*
+	 * REVISIT shouldn't most, or all, these zylinjtag_*()
+	 * entries be #ifdef ZY1000?  If not, why so they have
+	 * those names?
+	 *
+	 * Peek and poke are security holes -- they manipulate
+	 * server-internal addresses.
+	 */
 
-int ioutil_init(struct command_context_s *cmd_ctx)
+	// jim handlers
+	{
+		.name = "peek",
+		.mode = COMMAND_ANY,
+		.jim_handler = zylinjtag_Jim_Command_peek,
+		.help = "peek at a memory address",
+		.usage = "address",
+	},
+	{
+		.name = "poke",
+		.mode = COMMAND_ANY,
+		.jim_handler = zylinjtag_Jim_Command_poke,
+		.help = "poke at a memory address",
+		.usage = "address value",
+	},
+	{
+		.name = "ls",
+		.mode = COMMAND_ANY,
+		.jim_handler = zylinjtag_Jim_Command_ls,
+		.help = "show a listing of files",
+		.usage = "dirname",
+	},
+	{
+		.name = "mac",
+		.mode = COMMAND_ANY,
+		.jim_handler = zylinjtag_Jim_Command_mac,
+		.help = "show MAC address",
+	},
+	{
+		.name = "ip",
+		.jim_handler = zylinjtag_Jim_Command_ip,
+		.mode = COMMAND_ANY,
+		.help = "show IP address",
+	},
+	COMMAND_REGISTRATION_DONE
+};
+
+int ioutil_init(struct command_context *cmd_ctx)
 {
-	register_command(cmd_ctx, NULL, "rm", handle_rm_command, COMMAND_ANY,
-			"remove file");
-
-	register_command(cmd_ctx, NULL, "cat", handle_cat_command, COMMAND_ANY,
-			"display file content");
-
-	register_command(cmd_ctx, NULL, "trunc", handle_trunc_command, COMMAND_ANY,
-			"truncate a file to 0 size");
-
-	register_command(cmd_ctx, NULL, "cp", handle_cp_command,
-					 COMMAND_ANY, "copy a file <from> <to>");
-
-	register_command(cmd_ctx, NULL, "append_file", handle_append_command,
-			COMMAND_ANY, "append a variable number of strings to a file");
-
-	register_command(cmd_ctx, NULL, "meminfo", handle_meminfo_command,
-			COMMAND_ANY, "display available ram memory");
-
-    Jim_CreateCommand(interp, "rm", zylinjtag_Jim_Command_rm, NULL, NULL);
-
-    Jim_CreateCommand(interp, "peek", zylinjtag_Jim_Command_peek, NULL, NULL);
-    Jim_CreateCommand(interp, "poke", zylinjtag_Jim_Command_poke, NULL, NULL);
-    Jim_CreateCommand(interp, "ls", zylinjtag_Jim_Command_ls, NULL, NULL);
-
-	Jim_CreateCommand(interp, "mac", zylinjtag_Jim_Command_mac,
-			NULL, NULL);
-
-	Jim_CreateCommand(interp, "ip", zylinjtag_Jim_Command_ip,
-			NULL, NULL);
-
-    return ERROR_OK;
+	return register_commands(cmd_ctx, NULL, ioutil_command_handlers);
 }
-
-
