@@ -27,10 +27,9 @@
 #include "config.h"
 #endif
 
-#include "log.h"
 #include "time_support.h"
 // @todo the inclusion of server.h here is a layering violation
-#include "server.h"
+#include <server/server.h>
 
 #include <stdarg.h>
 
@@ -45,7 +44,7 @@
 int debug_level = -1;
 
 static FILE* log_output;
-static log_callback_t *log_callbacks = NULL;
+static struct log_callback *log_callbacks = NULL;
 
 static long long last_time;
 static long long current_time;
@@ -78,11 +77,11 @@ struct store_log_forward
 };
 
 /* either forward the log to the listeners or store it for possible forwarding later */
-static void log_forward(const char *file, int line, const char *function, const char *string)
+static void log_forward(const char *file, unsigned line, const char *function, const char *string)
 {
 	if (log_forward_count==0)
 	{
-		log_callback_t *cb, *next;
+		struct log_callback *cb, *next;
 		cb = log_callbacks;
 		/* DANGER!!!! the log callback can remove itself!!!! */
 		while (cb)
@@ -225,7 +224,7 @@ static void log_puts(enum log_levels level, const char *file, int line, const ch
 }
 
 
-void log_printf(enum log_levels level, const char *file, int line, const char *function, const char *format, ...)
+void log_printf(enum log_levels level, const char *file, unsigned line, const char *function, const char *format, ...)
 {
 	char *string;
 	va_list ap;
@@ -246,7 +245,7 @@ void log_printf(enum log_levels level, const char *file, int line, const char *f
 	va_end(ap);
 }
 
-void log_printf_lf(enum log_levels level, const char *file, int line, const char *function, const char *format, ...)
+void log_printf_lf(enum log_levels level, const char *file, unsigned line, const char *function, const char *format, ...)
 {
 	char *string;
 	va_list ap;
@@ -274,41 +273,42 @@ void log_printf_lf(enum log_levels level, const char *file, int line, const char
  * 2: + INFORMATIONAL MSGS
  * 3: + DEBUG MSGS
  */
-int handle_debug_level_command(struct command_context_s *cmd_ctx, char *cmd, char **args, int argc)
+COMMAND_HANDLER(handle_debug_level_command)
 {
-	if (argc == 1)
+	if (CMD_ARGC == 1)
 	{
 		unsigned new_level;
-		int retval = parse_uint(args[0], &new_level);
-		if (ERROR_OK != retval)
-			return retval;
+		COMMAND_PARSE_NUMBER(uint, CMD_ARGV[0], new_level);
 		debug_level = MIN(new_level, LOG_LVL_DEBUG);
 	}
-	else if (argc > 1)
+	else if (CMD_ARGC > 1)
 		return ERROR_COMMAND_SYNTAX_ERROR;
 
 	if (debug_level >= LOG_LVL_DEBUG && server_use_pipes == 1)
 	{
-		/* if we are enabling debug info then we need to write to a log file
-		 * otherwise the pipe will get full and cause issues with gdb */
+		/* if we are enabling debug info then we need to write to a
+		 * log file otherwise the pipe will get full and cause issues
+		 * with gdb
+		 */
 		FILE* file = fopen("openocd.log", "w");
 		if (file)
 		{
 			log_output = file;
-			LOG_WARNING("enabling log output as we are using pipes");
+			LOG_WARNING("enabling logfile output because "
+				"we are using pipes to talk to GDB.");
 		}
 	}
 
-	command_print(cmd_ctx, "debug_level: %i", debug_level);
+	command_print(CMD_CTX, "debug_level: %i", debug_level);
 
 	return ERROR_OK;
 }
 
-int handle_log_output_command(struct command_context_s *cmd_ctx, char *cmd, char **args, int argc)
+COMMAND_HANDLER(handle_log_output_command)
 {
-	if (argc == 1)
+	if (CMD_ARGC == 1)
 	{
-		FILE* file = fopen(args[0], "w");
+		FILE* file = fopen(CMD_ARGV[0], "w");
 
 		if (file)
 		{
@@ -319,34 +319,58 @@ int handle_log_output_command(struct command_context_s *cmd_ctx, char *cmd, char
 	return ERROR_OK;
 }
 
-int log_register_commands(struct command_context_s *cmd_ctx)
-{
-	start = timeval_ms();
-	register_command(cmd_ctx, NULL, "log_output", handle_log_output_command,
-		COMMAND_ANY, "redirect logging to <file> (default: stderr)");
-	register_command(cmd_ctx, NULL, "debug_level", handle_debug_level_command,
-		COMMAND_ANY, "adjust debug level <0-3>");
+static struct command_registration log_command_handlers[] = {
+	{
+		.name = "log_output",
+		.handler = handle_log_output_command,
+		.mode = COMMAND_ANY,
+		.help = "redirect logging to a file (default: stderr)",
+		.usage = "file_name",
+	},
+	{
+		.name = "debug_level",
+		.handler = handle_debug_level_command,
+		.mode = COMMAND_ANY,
+		.help = "Sets the verbosity level of debugging output. "
+			"0 shows errors only; 1 adds warnings; "
+			"2 (default) adds other info; 3 adds debugging.",
+		.usage = "number",
+	},
+	COMMAND_REGISTRATION_DONE
+};
 
-	return ERROR_OK;
+int log_register_commands(struct command_context *cmd_ctx)
+{
+	return register_commands(cmd_ctx, NULL, log_command_handlers);
 }
 
-int log_init(struct command_context_s *cmd_ctx)
+void log_init(void)
 {
-	/* set defaults for daemon configuration, if not set by cmdline or cfgfile */
+	/* set defaults for daemon configuration,
+	 * if not set by cmdline or cfgfile */
 	if (debug_level == -1)
 		debug_level = LOG_LVL_INFO;
 
-	if (log_output == NULL)
+	char *debug_env = getenv("OPENOCD_DEBUG_LEVEL");
+	if (NULL != debug_env)
 	{
-		log_output = stderr;
+		int value;
+		int retval = parse_int(debug_env, &value);
+		if (ERROR_OK == retval &&
+				debug_level >= LOG_LVL_SILENT &&
+				debug_level <= LOG_LVL_DEBUG)
+		{
+			debug_level = value;
+		}
 	}
 
-	start = last_time = timeval_ms();
+	if (log_output == NULL)
+		log_output = stderr;
 
-	return ERROR_OK;
+	start = last_time = timeval_ms();
 }
 
-int set_log_output(struct command_context_s *cmd_ctx, FILE *output)
+int set_log_output(struct command_context *cmd_ctx, FILE *output)
 {
 	log_output = output;
 	return ERROR_OK;
@@ -355,7 +379,7 @@ int set_log_output(struct command_context_s *cmd_ctx, FILE *output)
 /* add/remove log callback handler */
 int log_add_callback(log_callback_fn fn, void *priv)
 {
-	log_callback_t *cb;
+	struct log_callback *cb;
 
 	/* prevent the same callback to be registered more than once, just for sure */
 	for (cb = log_callbacks; cb; cb = cb->next)
@@ -365,7 +389,7 @@ int log_add_callback(log_callback_fn fn, void *priv)
 	}
 
 	/* alloc memory, it is safe just to return in case of an error, no need for the caller to check this */
-	if ((cb = malloc(sizeof(log_callback_t))) == NULL)
+	if ((cb = malloc(sizeof(struct log_callback))) == NULL)
 		return ERROR_BUF_TOO_SMALL;
 
 	/* add item to the beginning of the linked list */
@@ -379,7 +403,7 @@ int log_add_callback(log_callback_fn fn, void *priv)
 
 int log_remove_callback(log_callback_fn fn, void *priv)
 {
-	log_callback_t *cb, **p;
+	struct log_callback *cb, **p;
 
 	for (p = &log_callbacks; (cb = *p); p = &(*p)->next)
 	{
@@ -398,37 +422,26 @@ int log_remove_callback(log_callback_fn fn, void *priv)
 /* return allocated string w/printf() result */
 char *alloc_vprintf(const char *fmt, va_list ap)
 {
-	/* no buffer at the beginning, force realloc to do the job */
-	char *string = NULL;
+	va_list ap_copy;
+	int len;
+	char *string;
 
-	/* start with buffer size suitable for typical messages */
-	int size = 128;
+	/* determine the length of the buffer needed */
+	va_copy(ap_copy, ap);
+	len = vsnprintf(NULL, 0, fmt, ap_copy);
+	va_end(ap_copy);
 
-	for (;;)
-	{
-		char *t = string;
-		va_list ap_copy;
-		int ret;
-		string = realloc(string, size);
-		if (string == NULL)
-		{
-			if (t != NULL)
-				free(t);
-			return NULL;
-		}
+	/* allocate and make room for terminating zero. */
+	/* FIXME: The old version always allocated at least one byte extra and
+	 * other code depend on that. They should be probably be fixed, but for
+	 * now reserve the extra byte. */
+	string = malloc(len + 2);
+	if (string == NULL)
+		return NULL;
 
-		va_copy(ap_copy, ap);
+	/* do the real work */
+	vsnprintf(string, len + 1, fmt, ap);
 
-		ret = vsnprintf(string, size, fmt, ap_copy);
-		/* NB! The result of the vsnprintf() might be an *EMPTY* string! */
-		if ((ret >= 0) && ((ret + 1) < size))
-			break;
-
-		/* there was just enough or not enough space, allocate more in the next round */
-		size *= 2; /* double the buffer size */
-	}
-
-	/* the returned buffer is by principle guaranteed to be at least one character longer */
 	return string;
 }
 
@@ -507,27 +520,24 @@ void kept_alive()
 }
 
 /* if we sleep for extended periods of time, we must invoke keep_alive() intermittantly */
-void alive_sleep(int ms)
+void alive_sleep(uint64_t ms)
 {
-	int i;
-	int napTime = 10;
-	for (i = 0; i < ms; i += napTime)
+	uint64_t napTime = 10;
+	for (uint64_t i = 0; i < ms; i += napTime)
 	{
-		int sleep_a_bit = ms-i;
+		uint64_t sleep_a_bit = ms - i;
 		if (sleep_a_bit > napTime)
-		{
 			sleep_a_bit = napTime;
-		}
-		usleep(sleep_a_bit*1000);
+
+		usleep(sleep_a_bit * 1000);
 		keep_alive();
 	}
 }
 
-void busy_sleep(int ms)
+void busy_sleep(uint64_t ms)
 {
-	long long then;
-	then = timeval_ms();
-	while ((timeval_ms()-then) < ms)
+	uint64_t then = timeval_ms();
+	while (timeval_ms() - then < ms)
 	{
 		/* busy wait */
 	}
