@@ -26,7 +26,7 @@
 #ifndef ERROR_H
 #define ERROR_H
 
-#include "command.h"
+#include <helper/command.h>
 
 /* To achieve C99 printf compatibility in MinGW, gnu_printf should be
  * used for __attribute__((format( ... ))), with GCC v4.4 or later
@@ -59,20 +59,26 @@ enum log_levels
 	LOG_LVL_DEBUG = 3
 };
 
-extern void log_printf(enum log_levels level, const char *file, int line,
+void log_printf(enum log_levels level, const char *file, unsigned line,
 	const char *function, const char *format, ...)
 __attribute__ ((format (PRINTF_ATTRIBUTE_FORMAT, 5, 6)));
-extern void log_printf_lf(enum log_levels level, const char *file, int line,
+void log_printf_lf(enum log_levels level, const char *file, unsigned line,
 	const char *function, const char *format, ...)
 __attribute__ ((format (PRINTF_ATTRIBUTE_FORMAT, 5, 6)));
-extern int log_register_commands(struct command_context_s *cmd_ctx);
-extern int log_init(struct command_context_s *cmd_ctx);
-extern int set_log_output(struct command_context_s *cmd_ctx, FILE *output);
-extern void keep_alive(void);
-extern void kept_alive(void);
-extern void alive_sleep(int ms);
-extern void busy_sleep(int ms);
 
+/**
+ * Initialize logging module.  Call during program startup.
+ */
+void log_init(void);
+int set_log_output(struct command_context *cmd_ctx, FILE *output);
+
+int log_register_commands(struct command_context *cmd_ctx);
+
+void keep_alive(void);
+void kept_alive(void);
+
+void alive_sleep(uint64_t ms);
+void busy_sleep(uint64_t ms);
 
 /* log entries can be paused and replayed roughly according to the try/catch/rethrow
  * concepts in C++
@@ -82,18 +88,17 @@ void log_catch(void);
 void log_rethrow(void);
 
 
-typedef void (*log_callback_fn)(void *priv, const char *file, int line,
+typedef void (*log_callback_fn)(void *priv, const char *file, unsigned line,
 		const char *function, const char *string);
 
-typedef struct log_callback_s
-{
+struct log_callback {
 	log_callback_fn fn;
 	void *priv;
-	struct log_callback_s *next;
-} log_callback_t;
+	struct log_callback *next;
+};
 
-extern int log_add_callback(log_callback_fn fn, void *priv);
-extern int log_remove_callback(log_callback_fn fn, void *priv);
+int log_add_callback(log_callback_fn fn, void *priv);
+int log_remove_callback(log_callback_fn fn, void *priv);
 
 char *alloc_vprintf(const char *fmt, va_list ap);
 char *alloc_printf(const char *fmt, ...);
@@ -106,7 +111,12 @@ extern int debug_level;
 #define LOG_LEVEL_IS(FOO)  ((debug_level) >= (FOO))
 
 #define LOG_DEBUG(expr ...) \
-		((debug_level >= LOG_LVL_DEBUG) ? log_printf_lf (LOG_LVL_DEBUG, __FILE__, __LINE__, __FUNCTION__, expr) , 0 : 0)
+		do { \
+			if (debug_level >= LOG_LVL_DEBUG) \
+				log_printf_lf(LOG_LVL_DEBUG, \
+					__FILE__, __LINE__, __func__, \
+					expr); \
+		} while (0)
 
 #define LOG_INFO(expr ...) \
 		log_printf_lf (LOG_LVL_INFO, __FILE__, __LINE__, __FUNCTION__, expr)
