@@ -28,8 +28,6 @@
 #endif
 
 #include "time_support.h"
-#include "log.h"
-
 
 /* calculate difference between two struct timeval values */
 int timeval_subtract(struct timeval *result, struct timeval *x, struct timeval *y)
@@ -56,20 +54,11 @@ int timeval_subtract(struct timeval *result, struct timeval *x, struct timeval *
 /* add two struct timeval values */
 int timeval_add(struct timeval *result, struct timeval *x, struct timeval *y)
 {
-	result->tv_sec = x->tv_sec + y->tv_sec;
-
-	result->tv_usec = x->tv_usec + y->tv_usec;
-
-	while (result->tv_usec > 1000000)
-	{
-		result->tv_usec -= 1000000;
-		result->tv_sec++;
-	}
-
-	return 0;
+	memcpy(result, x, sizeof(struct timeval));
+	return timeval_add_time(result, y->tv_sec, y->tv_usec);
 }
 
-int timeval_add_time(struct timeval *result, int sec, int usec)
+int timeval_add_time(struct timeval *result, long sec, long usec)
 {
 	result->tv_sec += sec;
 	result->tv_usec += usec;
@@ -83,39 +72,28 @@ int timeval_add_time(struct timeval *result, int sec, int usec)
 	return 0;
 }
 
-void duration_start_measure(duration_t *duration)
+int duration_start(struct duration *duration)
 {
-	gettimeofday(&duration->start, NULL);
+	return gettimeofday(&duration->start, NULL);
 }
 
-int duration_stop_measure(duration_t *duration, char **text)
+int duration_measure(struct duration *duration)
 {
 	struct timeval end;
-
-	gettimeofday(&end, NULL);
-
-	timeval_subtract(&duration->duration, &end, &duration->start);
-
-	if (text)
-	{
-		float t;
-		t = duration->duration.tv_sec;
-		t += (float)duration->duration.tv_usec/1000000.0;
-		*text = malloc(100);
-		snprintf(*text, 100, "%fs", t);
-	}
-
-	return ERROR_OK;
+	int retval = gettimeofday(&end, NULL);
+	if (0 == retval)
+		timeval_subtract(&duration->elapsed, &end, &duration->start);
+	return retval;
 }
 
-long long timeval_ms()
+float duration_elapsed(struct duration *duration)
 {
-	struct timeval now;
-	long long t = 0;
-	gettimeofday(&now, NULL);
-
-	t += now.tv_usec/1000;
-	t += now.tv_sec*1000;
-
+	float t = duration->elapsed.tv_sec;
+	t += (float)duration->elapsed.tv_usec / 1000000.0;
 	return t;
 }
+
+float duration_kbps(struct duration *duration, size_t count)
+{
+	return count / (1024.0 * duration_elapsed(duration));
+}
