@@ -23,6 +23,7 @@
 #ifndef TYPES_H
 #define TYPES_H
 
+#include <stddef.h>
 #ifdef HAVE_SYS_TYPES_H
 #include <sys/types.h>
 #endif
@@ -33,8 +34,6 @@
 #include <inttypes.h>
 #endif
 
-typedef struct jtag_tap_s jtag_tap_t;
-
 #ifdef HAVE_STDBOOL_H
 #include <stdbool.h>
 #else	/* HAVE_STDBOOL_H */
@@ -56,6 +55,43 @@ typedef bool _Bool;
 
 #endif	/* HAVE_STDBOOL_H */
 
+/// turns a macro argument into a string constant
+#define stringify(s) __stringify(s)
+#define __stringify(s) #s
+
+
+/**
+ * Compute the number of elements of a variable length array.
+ * <code>
+ * const char *strs[] = { "a", "b", "c" };
+ * unsigned num_strs = ARRAY_SIZE(strs);
+ * </code>
+ */
+#define ARRAY_SIZE(x) (sizeof(x) / sizeof(*(x)))
+
+
+/**
+ * Cast a member of a structure out to the containing structure.
+ * @param ptr The pointer to the member.
+ * @param type The type of the container struct this is embedded in.
+ * @param member The name of the member within the struct.
+ *
+ * This is a mechanism which is used throughout the Linux kernel.
+ */
+#define container_of(ptr, type, member) ({			\
+	const typeof( ((type *)0)->member ) *__mptr = (ptr);	\
+	(type *)( (char *)__mptr - offsetof(type,member) );})
+
+
+/**
+ * Rounds @c m up to the nearest multiple of @c n using division.
+ * @param m The value to round up to @c n.
+ * @param n Round @c m up to a multiple of this number.
+ * @returns The rounded integer value.
+ */
+#define DIV_ROUND_UP(m, n)	(((m) + (n) - 1) / (n))
+
+
 /* DANGER!!!! here be dragons!
  *
  * Leave these fn's as byte accesses because it is safe
@@ -136,6 +172,7 @@ static inline void h_u16_to_be(uint8_t* buf, int val)
 #define SCNx32 "x"
 #define PRIi32 "i"
 #define PRIu32 "u"
+#define PRId8 PRId32
 
 typedef CYG_ADDRWORD intptr_t;
 typedef int64_t intmax_t;
