@@ -31,27 +31,27 @@
 #include "config.h"
 #endif
 
+#include <jtag/jtag.h>
 #include "commands.h"
 
-typedef struct cmd_queue_page_s
-{
+struct cmd_queue_page {
 	void *address;
 	size_t used;
-	struct cmd_queue_page_s *next;
-} cmd_queue_page_t;
+	struct cmd_queue_page *next;
+};
 
 #define CMD_QUEUE_PAGE_SIZE (1024 * 1024)
-static cmd_queue_page_t *cmd_queue_pages = NULL;
+static struct cmd_queue_page *cmd_queue_pages = NULL;
 
-jtag_command_t *jtag_command_queue = NULL;
-static jtag_command_t **next_command_pointer = &jtag_command_queue;
+struct jtag_command *jtag_command_queue = NULL;
+static struct jtag_command **next_command_pointer = &jtag_command_queue;
 
-void jtag_queue_command(jtag_command_t * cmd)
+void jtag_queue_command(struct jtag_command * cmd)
 {
 	// this command goes on the end, so ensure the queue terminates
 	cmd->next = NULL;
 
-	jtag_command_t **last_cmd = next_command_pointer;
+	struct jtag_command **last_cmd = next_command_pointer;
 	assert(NULL != last_cmd);
 	assert(NULL == *last_cmd);
 	*last_cmd = cmd;
@@ -62,7 +62,7 @@ void jtag_queue_command(jtag_command_t * cmd)
 
 void* cmd_queue_alloc(size_t size)
 {
-	cmd_queue_page_t **p_page = &cmd_queue_pages;
+	struct cmd_queue_page **p_page = &cmd_queue_pages;
 	int offset;
 	uint8_t *t;
 
@@ -108,7 +108,7 @@ void* cmd_queue_alloc(size_t size)
 
 	if (!*p_page)
 	{
-		*p_page = malloc(sizeof(cmd_queue_page_t));
+		*p_page = malloc(sizeof(struct cmd_queue_page));
 		(*p_page)->used = 0;
 		(*p_page)->address = malloc(CMD_QUEUE_PAGE_SIZE);
 		(*p_page)->next = NULL;
@@ -123,11 +123,11 @@ void* cmd_queue_alloc(size_t size)
 
 void cmd_queue_free(void)
 {
-	cmd_queue_page_t *page = cmd_queue_pages;
+	struct cmd_queue_page *page = cmd_queue_pages;
 
 	while (page)
 	{
-		cmd_queue_page_t *last = page;
+		struct cmd_queue_page *last = page;
 		free(page->address);
 		page = page->next;
 		free(last);
@@ -144,7 +144,7 @@ void jtag_command_queue_reset(void)
 	next_command_pointer = &jtag_command_queue;
 }
 
-enum scan_type jtag_scan_type(const scan_command_t *cmd)
+enum scan_type jtag_scan_type(const struct scan_command *cmd)
 {
 	int i;
 	int type = 0;
@@ -160,7 +160,7 @@ enum scan_type jtag_scan_type(const scan_command_t *cmd)
 	return type;
 }
 
-int jtag_scan_size(const scan_command_t *cmd)
+int jtag_scan_size(const struct scan_command *cmd)
 {
 	int bit_count = 0;
 	int i;
@@ -174,13 +174,13 @@ int jtag_scan_size(const scan_command_t *cmd)
 	return bit_count;
 }
 
-int jtag_build_buffer(const scan_command_t *cmd, uint8_t **buffer)
+int jtag_build_buffer(const struct scan_command *cmd, uint8_t **buffer)
 {
 	int bit_count = 0;
 	int i;
 
 	bit_count = jtag_scan_size(cmd);
-	*buffer = calloc(1,CEIL(bit_count, 8));
+	*buffer = calloc(1,DIV_ROUND_UP(bit_count, 8));
 
 	bit_count = 0;
 
@@ -219,7 +219,7 @@ int jtag_build_buffer(const scan_command_t *cmd, uint8_t **buffer)
 	return bit_count;
 }
 
-int jtag_read_buffer(uint8_t *buffer, const scan_command_t *cmd)
+int jtag_read_buffer(uint8_t *buffer, const struct scan_command *cmd)
 {
 	int i;
 	int bit_count = 0;
@@ -236,7 +236,7 @@ int jtag_read_buffer(uint8_t *buffer, const scan_command_t *cmd)
 		if (cmd->fields[i].in_value)
 		{
 			int num_bits = cmd->fields[i].num_bits;
-			uint8_t *captured = buf_set_buf(buffer, bit_count, malloc(CEIL(num_bits, 8)), 0, num_bits);
+			uint8_t *captured = buf_set_buf(buffer, bit_count, malloc(DIV_ROUND_UP(num_bits, 8)), 0, num_bits);
 
 #ifdef _DEBUG_JTAG_IO_
 			char *char_buf = buf_to_str(captured,
