@@ -32,7 +32,6 @@
 #endif
 
 #include "jtag.h"
-#include "minidriver.h"
 #include "interface.h"
 
 #ifdef HAVE_STRINGS_H
@@ -43,8 +42,8 @@
 /// The number of JTAG queue flushes (for profiling and debugging purposes).
 static int jtag_flush_queue_count;
 
-static void jtag_add_scan_check(void (*jtag_add_scan)(int in_num_fields, const scan_field_t *in_fields, tap_state_t state),
-		int in_num_fields, scan_field_t *in_fields, tap_state_t state);
+static void jtag_add_scan_check(void (*jtag_add_scan)(int in_num_fields, const struct scan_field *in_fields, tap_state_t state),
+		int in_num_fields, struct scan_field *in_fields, tap_state_t state);
 
 /**
  * The jtag_error variable is set when an error occurs while executing
@@ -78,7 +77,7 @@ static int jtag_srst = -1;
 /**
  * List all TAPs that have been created.
  */
-static jtag_tap_t *__jtag_all_taps = NULL;
+static struct jtag_tap *__jtag_all_taps = NULL;
 /**
  * The number of TAPs in the __jtag_all_taps list, used to track the
  * assigned chain position to new TAPs
@@ -98,15 +97,21 @@ static int jtag_ntrst_delay = 0; /* default to no nTRST delay */
 static int jtag_nsrst_assert_width = 0; /* width of assertion */
 static int jtag_ntrst_assert_width = 0; /* width of assertion */
 
-typedef struct jtag_event_callback_s
-{
-	jtag_event_handler_t          callback;
-	void*                         priv;
-	struct jtag_event_callback_s* next;
-} jtag_event_callback_t;
+/**
+ * Contains a single callback along with a pointer that will be passed
+ * when an event occurs.
+ */
+struct jtag_event_callback {
+	/// a event callback
+	jtag_event_handler_t callback;
+	/// the private data to pass to the callback
+	void* priv;
+	/// the next callback
+	struct jtag_event_callback* next;
+};
 
 /* callbacks to inform high-level handlers about JTAG state changes */
-static jtag_event_callback_t *jtag_event_callbacks;
+static struct jtag_event_callback *jtag_event_callbacks;
 
 /* speed in kHz*/
 static int speed_khz = 0;
@@ -115,10 +120,10 @@ static int rclk_fallback_speed_khz = 0;
 static enum {CLOCK_MODE_SPEED, CLOCK_MODE_KHZ, CLOCK_MODE_RCLK} clock_mode;
 static int jtag_speed = 0;
 
-static struct jtag_interface_s *jtag = NULL;
+static struct jtag_interface *jtag = NULL;
 
 /* configuration */
-jtag_interface_t *jtag_interface = NULL;
+struct jtag_interface *jtag_interface = NULL;
 
 void jtag_set_error(int error)
 {
@@ -164,7 +169,7 @@ void jtag_poll_set_enabled(bool value)
 
 /************/
 
-jtag_tap_t *jtag_all_taps(void)
+struct jtag_tap *jtag_all_taps(void)
 {
 	return __jtag_all_taps;
 };
@@ -176,7 +181,7 @@ unsigned jtag_tap_count(void)
 
 unsigned jtag_tap_count_enabled(void)
 {
-	jtag_tap_t *t = jtag_all_taps();
+	struct jtag_tap *t = jtag_all_taps();
 	unsigned n = 0;
 	while (t)
 	{
@@ -188,20 +193,20 @@ unsigned jtag_tap_count_enabled(void)
 }
 
 /// Append a new TAP to the chain of all taps.
-void jtag_tap_add(struct jtag_tap_s *t)
+void jtag_tap_add(struct jtag_tap *t)
 {
 	t->abs_chain_position = jtag_num_taps++;
 
-	jtag_tap_t **tap = &__jtag_all_taps;
+	struct jtag_tap **tap = &__jtag_all_taps;
 	while (*tap != NULL)
 		tap = &(*tap)->next_tap;
 	*tap = t;
 }
 
 /* returns a pointer to the n-th device in the scan chain */
-static inline jtag_tap_t *jtag_tap_by_position(unsigned n)
+static inline struct jtag_tap *jtag_tap_by_position(unsigned n)
 {
-	jtag_tap_t *t = jtag_all_taps();
+	struct jtag_tap *t = jtag_all_taps();
 
 	while (t && n-- > 0)
 		t = t->next_tap;
@@ -209,10 +214,10 @@ static inline jtag_tap_t *jtag_tap_by_position(unsigned n)
 	return t;
 }
 
-jtag_tap_t *jtag_tap_by_string(const char *s)
+struct jtag_tap *jtag_tap_by_string(const char *s)
 {
 	/* try by name first */
-	jtag_tap_t *t = jtag_all_taps();
+	struct jtag_tap *t = jtag_all_taps();
 
 	while (t)
 	{
@@ -238,18 +243,7 @@ jtag_tap_t *jtag_tap_by_string(const char *s)
 	return t;
 }
 
-jtag_tap_t *jtag_tap_by_jim_obj(Jim_Interp *interp, Jim_Obj *o)
-{
-	const char *cp = Jim_GetString(o, NULL);
-	jtag_tap_t *t = cp ? jtag_tap_by_string(cp) : NULL;
-	if (NULL == cp)
-		cp = "(unknown)";
-	if (NULL == t)
-		Jim_SetResult_sprintf(interp, "Tap '%s' could not be found", cp);
-	return t;
-}
-
-jtag_tap_t* jtag_tap_next_enabled(jtag_tap_t* p)
+struct jtag_tap* jtag_tap_next_enabled(struct jtag_tap* p)
 {
 	p = p ? p->next_tap : jtag_all_taps();
 	while (p)
@@ -261,7 +255,7 @@ jtag_tap_t* jtag_tap_next_enabled(jtag_tap_t* p)
 	return NULL;
 }
 
-const char *jtag_tap_name(const jtag_tap_t *tap)
+const char *jtag_tap_name(const struct jtag_tap *tap)
 {
 	return (tap == NULL) ? "(unknown)" : tap->dotted_name;
 }
@@ -269,7 +263,7 @@ const char *jtag_tap_name(const jtag_tap_t *tap)
 
 int jtag_register_event_callback(jtag_event_handler_t callback, void *priv)
 {
-	jtag_event_callback_t **callbacks_p = &jtag_event_callbacks;
+	struct jtag_event_callback **callbacks_p = &jtag_event_callbacks;
 
 	if (callback == NULL)
 	{
@@ -283,7 +277,7 @@ int jtag_register_event_callback(jtag_event_handler_t callback, void *priv)
 		callbacks_p = &((*callbacks_p)->next);
 	}
 
-	(*callbacks_p) = malloc(sizeof(jtag_event_callback_t));
+	(*callbacks_p) = malloc(sizeof(struct jtag_event_callback));
 	(*callbacks_p)->callback = callback;
 	(*callbacks_p)->priv = priv;
 	(*callbacks_p)->next = NULL;
@@ -293,8 +287,8 @@ int jtag_register_event_callback(jtag_event_handler_t callback, void *priv)
 
 int jtag_unregister_event_callback(jtag_event_handler_t callback, void *priv)
 {
-	jtag_event_callback_t **callbacks_p;
-	jtag_event_callback_t **next;
+	struct jtag_event_callback **callbacks_p;
+	struct jtag_event_callback **next;
 
 	if (callback == NULL)
 	{
@@ -322,13 +316,13 @@ int jtag_unregister_event_callback(jtag_event_handler_t callback, void *priv)
 
 int jtag_call_event_callbacks(enum jtag_event event)
 {
-	jtag_event_callback_t *callback = jtag_event_callbacks;
+	struct jtag_event_callback *callback = jtag_event_callbacks;
 
 	LOG_DEBUG("jtag event: %s", jtag_event_strings[event]);
 
 	while (callback)
 	{
-		jtag_event_callback_t *next;
+		struct jtag_event_callback *next;
 
 		/* callback may remove itself */
 		next = callback->next;
@@ -353,12 +347,12 @@ static void jtag_prelude(tap_state_t state)
 	cmd_queue_cur_state = state;
 }
 
-void jtag_alloc_in_value32(scan_field_t *field)
+void jtag_alloc_in_value32(struct scan_field *field)
 {
 	interface_jtag_alloc_in_value32(field);
 }
 
-void jtag_add_ir_scan_noverify(int in_count, const scan_field_t *in_fields,
+void jtag_add_ir_scan_noverify(int in_count, const struct scan_field *in_fields,
 		tap_state_t state)
 {
 	jtag_prelude(state);
@@ -368,7 +362,7 @@ void jtag_add_ir_scan_noverify(int in_count, const scan_field_t *in_fields,
 }
 
 
-void jtag_add_ir_scan(int in_num_fields, scan_field_t *in_fields, tap_state_t state)
+void jtag_add_ir_scan(int in_num_fields, struct scan_field *in_fields, tap_state_t state)
 {
 	assert(state != TAP_RESET);
 
@@ -391,7 +385,7 @@ void jtag_add_ir_scan(int in_num_fields, scan_field_t *in_fields, tap_state_t st
 	}
 }
 
-void jtag_add_plain_ir_scan(int in_num_fields, const scan_field_t *in_fields,
+void jtag_add_plain_ir_scan(int in_num_fields, const struct scan_field *in_fields,
 		tap_state_t state)
 {
 	assert(state != TAP_RESET);
@@ -403,18 +397,6 @@ void jtag_add_plain_ir_scan(int in_num_fields, const scan_field_t *in_fields,
 	jtag_set_error(retval);
 }
 
-void jtag_add_callback(jtag_callback1_t f, jtag_callback_data_t data0)
-{
-	interface_jtag_add_callback(f, data0);
-}
-
-void jtag_add_callback4(jtag_callback_t f, jtag_callback_data_t data0,
-		jtag_callback_data_t data1, jtag_callback_data_t data2,
-		jtag_callback_data_t data3)
-{
-	interface_jtag_add_callback4(f, data0, data1, data2, data3);
-}
-
 static int jtag_check_value_inner(uint8_t *captured, uint8_t *in_check_value,
 		uint8_t *in_check_mask, int num_bits);
 
@@ -423,12 +405,12 @@ static int jtag_check_value_mask_callback(jtag_callback_data_t data0, jtag_callb
 	return jtag_check_value_inner((uint8_t *)data0, (uint8_t *)data1, (uint8_t *)data2, (int)data3);
 }
 
-static void jtag_add_scan_check(void (*jtag_add_scan)(int in_num_fields, const scan_field_t *in_fields, tap_state_t state),
-		int in_num_fields, scan_field_t *in_fields, tap_state_t state)
+static void jtag_add_scan_check(void (*jtag_add_scan)(int in_num_fields, const struct scan_field *in_fields, tap_state_t state),
+		int in_num_fields, struct scan_field *in_fields, tap_state_t state)
 {
 	for (int i = 0; i < in_num_fields; i++)
 	{
-		struct scan_field_s *field = &in_fields[i];
+		struct scan_field *field = &in_fields[i];
 		field->allocated = 0;
 		field->modified = 0;
 		if (field->check_value || field->in_value)
@@ -460,7 +442,7 @@ static void jtag_add_scan_check(void (*jtag_add_scan)(int in_num_fields, const s
 	}
 }
 
-void jtag_add_dr_scan_check(int in_num_fields, scan_field_t *in_fields, tap_state_t state)
+void jtag_add_dr_scan_check(int in_num_fields, struct scan_field *in_fields, tap_state_t state)
 {
 	if (jtag_verify)
 	{
@@ -472,7 +454,7 @@ void jtag_add_dr_scan_check(int in_num_fields, scan_field_t *in_fields, tap_stat
 }
 
 
-void jtag_add_dr_scan(int in_num_fields, const scan_field_t *in_fields,
+void jtag_add_dr_scan(int in_num_fields, const struct scan_field *in_fields,
 		tap_state_t state)
 {
 	assert(state != TAP_RESET);
@@ -484,7 +466,7 @@ void jtag_add_dr_scan(int in_num_fields, const scan_field_t *in_fields,
 	jtag_set_error(retval);
 }
 
-void jtag_add_plain_dr_scan(int in_num_fields, const scan_field_t *in_fields,
+void jtag_add_plain_dr_scan(int in_num_fields, const struct scan_field *in_fields,
 		tap_state_t state)
 {
 	assert(state != TAP_RESET);
@@ -496,20 +478,6 @@ void jtag_add_plain_dr_scan(int in_num_fields, const scan_field_t *in_fields,
 	jtag_set_error(retval);
 }
 
-void jtag_add_dr_out(jtag_tap_t* tap,
-		int num_fields, const int* num_bits, const uint32_t* value,
-		tap_state_t end_state)
-{
-	assert(end_state != TAP_RESET);
-	assert(end_state != TAP_INVALID);
-
-	cmd_queue_cur_state = end_state;
-
-	interface_jtag_add_dr_out(tap,
-			num_fields, num_bits, value,
-			end_state);
-}
-
 void jtag_add_tlr(void)
 {
 	jtag_prelude(TAP_RESET);
@@ -562,10 +530,12 @@ int jtag_add_statemove(tap_state_t goal_state)
 {
 	tap_state_t cur_state = cmd_queue_cur_state;
 
-	LOG_DEBUG("cur_state=%s goal_state=%s",
-		tap_state_name(cur_state),
-		tap_state_name(goal_state));
-
+	if (goal_state != cur_state)
+	{
+		LOG_DEBUG("cur_state=%s goal_state=%s",
+			tap_state_name(cur_state),
+			tap_state_name(goal_state));
+	}
 
 	/* If goal is RESET, be paranoid and force that that transition
 	 * (e.g. five TCK cycles, TMS high).  Else trust "cur_state".
@@ -580,7 +550,7 @@ int jtag_add_statemove(tap_state_t goal_state)
 		unsigned tms_bits  = tap_get_tms_path(cur_state, goal_state);
 		unsigned tms_count = tap_get_tms_path_len(cur_state, goal_state);
 		tap_state_t moves[8];
-		assert(tms_count < DIM(moves));
+		assert(tms_count < ARRAY_SIZE(moves));
 
 		for (unsigned i = 0; i < tms_count; i++, tms_bits >>= 1)
 		{
@@ -767,8 +737,7 @@ static int jtag_check_value_inner(uint8_t *captured, uint8_t *in_check_value,
 		uint8_t *in_check_mask, int num_bits)
 {
 	int retval = ERROR_OK;
-
-	int compare_failed = 0;
+	int compare_failed;
 
 	if (in_check_mask)
 		compare_failed = buf_cmp_mask(captured, in_check_value, in_check_mask, num_bits);
@@ -806,7 +775,7 @@ static int jtag_check_value_inner(uint8_t *captured, uint8_t *in_check_value,
 	return retval;
 }
 
-void jtag_check_value_mask(scan_field_t *field, uint8_t *value, uint8_t *mask)
+void jtag_check_value_mask(struct scan_field *field, uint8_t *value, uint8_t *mask)
 {
 	assert(field->in_value != NULL);
 
@@ -856,7 +825,7 @@ int jtag_execute_queue(void)
 
 static int jtag_reset_callback(enum jtag_event event, void *priv)
 {
-	jtag_tap_t *tap = priv;
+	struct jtag_tap *tap = priv;
 
 	if (event == JTAG_TRST_ASSERTED)
 	{
@@ -895,7 +864,7 @@ void jtag_sleep(uint32_t us)
 
 static int jtag_examine_chain_execute(uint8_t *idcode_buffer, unsigned num_idcode)
 {
-	scan_field_t field = {
+	struct scan_field field = {
 			.tap = NULL,
 			.num_bits = num_idcode * 32,
 			.out_value = idcode_buffer,
@@ -989,18 +958,27 @@ static bool jtag_examine_chain_end(uint8_t *idcodes, unsigned count, unsigned ma
 	return triggered;
 }
 
-static bool jtag_examine_chain_match_tap(const struct jtag_tap_s *tap)
+static bool jtag_examine_chain_match_tap(const struct jtag_tap *tap)
 {
+	uint32_t idcode = tap->idcode;
+
 	/* ignore expected BYPASS codes; warn otherwise */
-	if (0 == tap->expected_ids_cnt && !tap->idcode)
+	if (0 == tap->expected_ids_cnt && !idcode)
 		return true;
 
+	/* optionally ignore the JTAG version field */
+	uint32_t mask = tap->ignore_version ? ~(0xff << 24) : ~0;
+
+	idcode &= mask;
+
 	/* Loop over the expected identification codes and test for a match */
 	unsigned ii, limit = tap->expected_ids_cnt;
 
 	for (ii = 0; ii < limit; ii++)
 	{
-		if (tap->idcode == tap->expected_ids[ii])
+		uint32_t expected = tap->expected_ids[ii] & mask;
+
+		if (idcode == expected)
 			return true;
 
 		/* treat "-expected-id 0" as a "don't-warn" wildcard */
@@ -1044,7 +1022,7 @@ static int jtag_examine_chain(void)
 		return ERROR_JTAG_INIT_FAILED;
 
 	/* point at the 1st tap */
-	jtag_tap_t *tap = jtag_tap_next_enabled(NULL);
+	struct jtag_tap *tap = jtag_tap_next_enabled(NULL);
 
 	if (!tap)
 		autoprobe = true;
@@ -1058,7 +1036,7 @@ static int jtag_examine_chain(void)
 		if ((idcode & 1) == 0)
 		{
 			/* Zero for LSB indicates a device in bypass */
-			LOG_WARNING("TAP %s does not have IDCODE",
+			LOG_INFO("TAP %s does not have IDCODE",
 					tap->dotted_name);
 			idcode = 0;
 			tap->hasidcode = false;
@@ -1168,10 +1146,10 @@ static int jtag_examine_chain(void)
  */
 static int jtag_validate_ircapture(void)
 {
-	jtag_tap_t *tap;
+	struct jtag_tap *tap;
 	int total_ir_length = 0;
 	uint8_t *ir_test = NULL;
-	scan_field_t field;
+	struct scan_field field;
 	int val;
 	int chain_pos = 0;
 	int retval;
@@ -1187,7 +1165,7 @@ static int jtag_validate_ircapture(void)
 	/* increase length to add 2 bit sentinel after scan */
 	total_ir_length += 2;
 
-	ir_test = malloc(CEIL(total_ir_length, 8));
+	ir_test = malloc(DIV_ROUND_UP(total_ir_length, 8));
 	if (ir_test == NULL)
 		return ERROR_FAIL;
 
@@ -1218,7 +1196,14 @@ static int jtag_validate_ircapture(void)
 		/* If we're autoprobing, guess IR lengths.  They must be at
 		 * least two bits.  Guessing will fail if (a) any TAP does
 		 * not conform to the JTAG spec; or (b) when the upper bits
-		 * captured from some conforming TAP are nonzero.
+		 * captured from some conforming TAP are nonzero.  Or if
+		 * (c) an IR length is longer than 32 bits -- which is only
+		 * an implementation limit, which could someday be raised.
+		 *
+		 * REVISIT optimization:  if there's a *single* TAP we can
+		 * lift restrictions (a) and (b) by scanning a recognizable
+		 * pattern before the all-ones BYPASS.  Check for where the
+		 * pattern starts in the result, instead of an 0...01 value.
 		 *
 		 * REVISIT alternative approach: escape to some tcl code
 		 * which could provide more knowledge, based on IDCODE; and
@@ -1227,7 +1212,8 @@ static int jtag_validate_ircapture(void)
 		if (tap->ir_length == 0) {
 			tap->ir_length = 2;
 			while ((val = buf_get_u32(ir_test, chain_pos,
-						tap->ir_length + 1)) == 1) {
+						tap->ir_length + 1)) == 1
+					&& tap->ir_length <= 32) {
 				tap->ir_length++;
 			}
 			LOG_WARNING("AUTO %s - use \"... -irlen %d\"",
@@ -1280,14 +1266,14 @@ done:
 }
 
 
-void jtag_tap_init(jtag_tap_t *tap)
+void jtag_tap_init(struct jtag_tap *tap)
 {
 	unsigned ir_len_bits;
 	unsigned ir_len_bytes;
 
 	/* if we're autoprobing, cope with potentially huge ir_length */
 	ir_len_bits = tap->ir_length ? : JTAG_IRLEN_MAX;
-	ir_len_bytes = CEIL(ir_len_bits, 8);
+	ir_len_bytes = DIV_ROUND_UP(ir_len_bits, 8);
 
 	tap->expected = calloc(1, ir_len_bytes);
 	tap->expected_mask = calloc(1, ir_len_bytes);
@@ -1315,7 +1301,7 @@ void jtag_tap_init(jtag_tap_t *tap)
 	jtag_tap_add(tap);
 }
 
-void jtag_tap_free(jtag_tap_t *tap)
+void jtag_tap_free(struct jtag_tap *tap)
 {
 	jtag_unregister_event_callback(&jtag_reset_callback, tap);
 
@@ -1328,7 +1314,7 @@ void jtag_tap_free(jtag_tap_t *tap)
 	free(tap);
 }
 
-int jtag_interface_init(struct command_context_s *cmd_ctx)
+int jtag_interface_init(struct command_context *cmd_ctx)
 {
 	if (jtag)
 		return ERROR_OK;
@@ -1369,9 +1355,9 @@ int jtag_interface_init(struct command_context_s *cmd_ctx)
 	return ERROR_OK;
 }
 
-int jtag_init_inner(struct command_context_s *cmd_ctx)
+int jtag_init_inner(struct command_context *cmd_ctx)
 {
-	jtag_tap_t *tap;
+	struct jtag_tap *tap;
 	int retval;
 	bool issue_setup = true;
 
@@ -1455,7 +1441,7 @@ int jtag_interface_quit(void)
 }
 
 
-int jtag_init_reset(struct command_context_s *cmd_ctx)
+int jtag_init_reset(struct command_context *cmd_ctx)
 {
 	int retval;
 
@@ -1509,7 +1495,7 @@ int jtag_init_reset(struct command_context_s *cmd_ctx)
 	return jtag_init_inner(cmd_ctx);
 }
 
-int jtag_init(struct command_context_s *cmd_ctx)
+int jtag_init(struct command_context *cmd_ctx)
 {
 	int retval;
 
@@ -1521,7 +1507,7 @@ int jtag_init(struct command_context_s *cmd_ctx)
 	if ((retval = jtag_execute_queue()) != ERROR_OK)
 		return retval;
 
-	if (Jim_Eval_Named(interp, "jtag_init", __FILE__, __LINE__) != JIM_OK)
+	if (Jim_Eval_Named(cmd_ctx->interp, "jtag_init", __FILE__, __LINE__) != JIM_OK)
 		return ERROR_FAIL;
 
 	return ERROR_OK;
@@ -1569,13 +1555,6 @@ static int jtag_set_speed(int speed)
 	return jtag ? jtag->speed(speed) : ERROR_OK;
 }
 
-int jtag_config_speed(int speed)
-{
-	LOG_DEBUG("handle jtag speed");
-	clock_mode = CLOCK_MODE_SPEED;
-	return jtag_set_speed(speed);
-}
-
 int jtag_config_khz(unsigned khz)
 {
 	LOG_DEBUG("handle jtag khz");
@@ -1644,6 +1623,13 @@ bool jtag_will_verify_capture_ir()
 
 int jtag_power_dropout(int *dropout)
 {
+	if (jtag == NULL)
+	{
+		/* TODO: as the jtag interface is not valid all
+		 * we can do at the moment is exit OpenOCD */
+		LOG_ERROR("No Valid JTAG Interface Configured.");
+		exit(-1);
+	}
 	return jtag->power_dropout(dropout);
 }
 
