@@ -363,7 +363,7 @@ const char *tap_state_name(tap_state_t state)
 {
 	unsigned i;
 
-	for (i = 0; i < DIM(tap_name_mapping); i++) {
+	for (i = 0; i < ARRAY_SIZE(tap_name_mapping); i++) {
 		if (tap_name_mapping[i].symbol == state)
 			return tap_name_mapping[i].name;
 	}
@@ -374,7 +374,7 @@ tap_state_t tap_state_by_name(const char *name)
 {
 	unsigned i;
 
-	for (i = 0; i < DIM(tap_name_mapping); i++) {
+	for (i = 0; i < ARRAY_SIZE(tap_name_mapping); i++) {
 		/* be nice to the human */
 		if (strcasecmp(name, tap_name_mapping[i].name) == 0)
 			return tap_name_mapping[i].symbol;
@@ -413,7 +413,7 @@ tap_state_t jtag_debug_state_machine(const void *tms_buf, const void *tdi_buf,
 	tms_buffer = (const uint8_t *)tms_buf;
 	tdi_buffer = (const uint8_t *)tdi_buf;
 
-	tap_bytes = TAP_SCAN_BYTES(tap_bits);
+	tap_bytes = DIV_ROUND_UP(tap_bits, 8);
 	DEBUG_JTAG_IO("TAP/SM: TMS bits: %u (bytes: %u)", tap_bits, tap_bytes);
 
 	tap_out_bits = 0;
