@@ -26,7 +26,7 @@
 #ifndef OPENOCD_JTAG_INTERFACE_H
 #define OPENOCD_JTAG_INTERFACE_H
 
-#include "jtag.h"
+#include <jtag/jtag.h>
 
 /* @file
  * The "Cable Helper API" is what the cable drivers can use to help
@@ -184,8 +184,7 @@ static inline tap_state_t jtag_debug_state_machine(const void *tms_buf,
 }
 #endif // _DEBUG_JTAG_IO_
 
-typedef struct jtag_interface_s
-{
+struct jtag_interface {
 	/// The name of the JTAG interface driver.
 	char* name;
 
@@ -205,10 +204,8 @@ typedef struct jtag_interface_s
 	/**
 	 * The interface driver may register additional commands to expose
 	 * additional features not covered by the standard command set.
-	 * @param cmd_ctx The context in which commands should be registered.
-	 * @returns ERROR_OK on success, or an error code on failure.
 	 */
-	int (*register_commands)(struct command_context_s* cmd_ctx);
+	const struct command_registration *commands;
 
 	/**
 	 * Interface driver must initalize any resources and connect to a
@@ -271,7 +268,7 @@ typedef struct jtag_interface_s
 	 * @returns ERROR_OK on success, or an error code on failure.
 	 */
 	int (*srst_asserted)(int* srst_asserted);
-} jtag_interface_t;
+};
 
 
 #endif // OPENOCD_JTAG_INTERFACE_H
