@@ -23,9 +23,8 @@
 #ifndef JTAG_H
 #define JTAG_H
 
-#include "binarybuffer.h"
-#include "log.h"
-
+#include <helper/binarybuffer.h>
+#include <helper/log.h>
 
 #ifdef _DEBUG_JTAG_IO_
 #define DEBUG_JTAG_IO(expr ...) \
@@ -39,17 +38,6 @@
 #define DEBUG_JTAG_IOZ 64
 #endif
 
-/*-----<Macros>--------------------------------------------------*/
-
-/**
- * When given an array, compute its DIMension; in other words, the
- * number of elements in the array
- */
-#define DIM(x)					(sizeof(x)/sizeof((x)[0]))
-
-/** Calculate the number of bytes required to hold @a n TAP scan bits */
-#define TAP_SCAN_BYTES(n)		CEIL(n, 8)
-
 /*-----</Macros>-------------------------------------------------*/
 
 /**
@@ -120,10 +108,9 @@ extern tap_state_t cmd_queue_cur_state;
  *
  * The allocated, modified, and intmp fields are internal work space.
  */
-typedef struct scan_field_s
-{
+struct scan_field {
 	/// A pointer to the tap structure to which this field refers.
-	jtag_tap_t* tap;
+	struct jtag_tap* tap;
 
 	/// The number of bits this field specifies (up to 32)
 	int num_bits;
@@ -143,15 +130,9 @@ typedef struct scan_field_s
 	int modified;
 	/// temporary storage for performing value checks synchronously
 	uint8_t intmp[4];
-} scan_field_t;
+};
 
-typedef struct jtag_tap_event_action_s jtag_tap_event_action_t;
-
-/* this is really: typedef jtag_tap_t */
-/* But - the typedef is done in "types.h" */
-/* due to "forward declaration reasons" */
-struct jtag_tap_s
-{
+struct jtag_tap {
 	const char* chip;
 	const char* tapname;
 	const char* dotted_name;
@@ -165,35 +146,39 @@ struct jtag_tap_s
 	uint8_t* expected; /**< Capture-IR expected value */
 	uint32_t ir_capture_mask;
 	uint8_t* expected_mask; /**< Capture-IR expected mask */
-	uint32_t idcode;
-	bool hasidcode; /* not all devices have idcode, we'll discover this during chain examination */
-	/**< device identification code */
+	uint32_t idcode; /**< device identification code */
+	/** not all devices have idcode,
+	 * we'll discover this during chain examination */
+	bool hasidcode;
 
 	/// Array of expected identification codes */
 	uint32_t* expected_ids;
 	/// Number of expected identification codes
 	uint8_t expected_ids_cnt;
 
+	/// Flag saying whether to ignore version field in expected_ids[]
+	bool ignore_version;
+
 	/// current instruction
 	uint8_t* cur_instr;
 	/// Bypass register selected
 	int bypass;
 
-	jtag_tap_event_action_t *event_action;
+	struct jtag_tap_event_action *event_action;
 
-	jtag_tap_t* next_tap;
+	struct jtag_tap* next_tap;
 };
 
-void jtag_tap_init(jtag_tap_t *tap);
-void jtag_tap_free(jtag_tap_t *tap);
+void jtag_tap_init(struct jtag_tap *tap);
+void jtag_tap_free(struct jtag_tap *tap);
 
-extern jtag_tap_t* jtag_all_taps(void);
-extern const char *jtag_tap_name(const jtag_tap_t *tap);
-extern jtag_tap_t* jtag_tap_by_string(const char* dotted_name);
-extern jtag_tap_t* jtag_tap_by_jim_obj(Jim_Interp* interp, Jim_Obj* obj);
-extern jtag_tap_t* jtag_tap_next_enabled(jtag_tap_t* p);
-extern unsigned jtag_tap_count_enabled(void);
-extern unsigned jtag_tap_count(void);
+struct jtag_tap* jtag_all_taps(void);
+const char *jtag_tap_name(const struct jtag_tap *tap);
+struct jtag_tap* jtag_tap_by_string(const char* dotted_name);
+struct jtag_tap* jtag_tap_by_jim_obj(Jim_Interp* interp, Jim_Obj* obj);
+struct jtag_tap* jtag_tap_next_enabled(struct jtag_tap* p);
+unsigned jtag_tap_count_enabled(void);
+unsigned jtag_tap_count(void);
 
 
 /*
@@ -225,11 +210,16 @@ enum jtag_event {
 	JTAG_TAP_EVENT_DISABLE,
 };
 
-struct jtag_tap_event_action_s
+struct jtag_tap_event_action
 {
-	enum jtag_event		event;
-	Jim_Obj*                 body;
-	jtag_tap_event_action_t* next;
+	/// The event for which this action will be triggered.
+	enum jtag_event event;
+	/// The interpreter to use for evaluating the @c body.
+	Jim_Interp *interp;
+	/// Contains a script to 'eval' when the @c event is triggered.
+	Jim_Obj *body;
+	// next action in linked list
+	struct jtag_tap_event_action *next;
 };
 
 /**
@@ -245,39 +235,33 @@ struct jtag_tap_event_action_s
  */
 typedef int (*jtag_event_handler_t)(enum jtag_event event, void* priv);
 
-extern int jtag_register_event_callback(jtag_event_handler_t f, void *x);
-extern int jtag_unregister_event_callback(jtag_event_handler_t f, void *x);
+int jtag_register_event_callback(jtag_event_handler_t f, void *x);
+int jtag_unregister_event_callback(jtag_event_handler_t f, void *x);
 
-extern int jtag_call_event_callbacks(enum jtag_event event);
+int jtag_call_event_callbacks(enum jtag_event event);
 
 
 /// @returns The current JTAG speed setting.
 int jtag_get_speed(void);
+
 /**
  * Given a @a speed setting, use the interface @c speed_div callback to
  * adjust the setting.
  * @param speed The speed setting to convert back to readable KHz.
  * @returns ERROR_OK if the interface has not been initialized or on success;
- * 	otherwise, the error code produced by the @c speed_div callback.
+ *	otherwise, the error code produced by the @c speed_div callback.
  */
 int jtag_get_speed_readable(int *speed);
-/**
- * Set the JTAG speed. This routine will call the underlying
- * interface @c speed callback, if the interface has been initialized.
- * @param speed The new speed setting.
- * @returns ERROR_OK during configuration or on success, or an error
- *   code returned from the interface @c speed callback.
- */
-int jtag_config_speed(int speed);
-
 
 /// Attempt to configure the interface for the specified KHz.
 int jtag_config_khz(unsigned khz);
+
 /**
  * Attempt to enable RTCK/RCLK. If that fails, fallback to the
  * specified frequency.
  */
 int jtag_config_rclk(unsigned fallback_speed_khz);
+
 /// Retreives the clock speed of the JTAG interface in KHz.
 unsigned jtag_get_speed_khz(void);
 
@@ -328,21 +312,21 @@ bool jtag_will_verify_capture_ir(void);
  * Initialize interface upon startup.  Return a successful no-op upon
  * subsequent invocations.
  */
-extern int  jtag_interface_init(struct command_context_s* cmd_ctx);
+int  jtag_interface_init(struct command_context* cmd_ctx);
 
 /// Shutdown the JTAG interface upon program exit.
-extern int  jtag_interface_quit(void);
+int  jtag_interface_quit(void);
 
 /**
  * Initialize JTAG chain using only a RESET reset. If init fails,
  * try reset + init.
  */
-extern int  jtag_init(struct command_context_s* cmd_ctx);
+int  jtag_init(struct command_context* cmd_ctx);
 
 /// reset, then initialize JTAG chain
-extern int jtag_init_reset(struct command_context_s* cmd_ctx);
-extern int jtag_register_commands(struct command_context_s* cmd_ctx);
-extern int jtag_init_inner(struct command_context_s *cmd_ctx);
+int jtag_init_reset(struct command_context* cmd_ctx);
+int jtag_register_commands(struct command_context* cmd_ctx);
+int jtag_init_inner(struct command_context *cmd_ctx);
 
 /**
  * @file
@@ -369,18 +353,21 @@ extern int jtag_init_inner(struct command_context_s *cmd_ctx);
  * subsequent DR SCANs.
  *
  */
-extern void jtag_add_ir_scan(int num_fields, scan_field_t* fields, tap_state_t endstate);
+void jtag_add_ir_scan(int num_fields,
+		struct scan_field* fields, tap_state_t endstate);
 /**
  * The same as jtag_add_ir_scan except no verification is performed out
  * the output values.
  */
-extern void jtag_add_ir_scan_noverify(int num_fields, const scan_field_t *fields, tap_state_t state);
+void jtag_add_ir_scan_noverify(int num_fields,
+		const struct scan_field *fields, tap_state_t state);
 /**
  * Duplicate the scan fields passed into the function into an IR SCAN
  * command.  This function assumes that the caller handles extra fields
  * for bypassed TAPs.
  */
-extern void jtag_add_plain_ir_scan(int num_fields, const scan_field_t* fields, tap_state_t endstate);
+void jtag_add_plain_ir_scan(int num_fields,
+		const struct scan_field* fields, tap_state_t endstate);
 
 
 /**
@@ -392,7 +379,7 @@ extern void jtag_add_plain_ir_scan(int num_fields, const scan_field_t* fields, t
  * allocation method is used, for the synchronous case the temporary 32
  * bits come from the input field itself.
  */
-extern void jtag_alloc_in_value32(scan_field_t *field);
+void jtag_alloc_in_value32(struct scan_field *field);
 
 /**
  * Generate a DR SCAN using the fields passed to the function.
@@ -400,15 +387,18 @@ extern void jtag_alloc_in_value32(scan_field_t *field);
  * specified there.  For bypassed TAPs, the function generates a dummy
  * 1-bit field.  The bypass status of TAPs is set by jtag_add_ir_scan().
  */
-extern void jtag_add_dr_scan(int num_fields, const scan_field_t* fields, tap_state_t endstate);
+void jtag_add_dr_scan(int num_fields,
+		const struct scan_field* fields, tap_state_t endstate);
 /// A version of jtag_add_dr_scan() that uses the check_value/mask fields
-extern void jtag_add_dr_scan_check(int num_fields, scan_field_t* fields, tap_state_t endstate);
+void jtag_add_dr_scan_check(int num_fields,
+		struct scan_field* fields, tap_state_t endstate);
 /**
  * Duplicate the scan fields passed into the function into a DR SCAN
  * command.  Unlike jtag_add_dr_scan(), this function assumes that the
  * caller handles extra fields for bypassed TAPs.
  */
-extern void jtag_add_plain_dr_scan(int num_fields, const scan_field_t* fields, tap_state_t endstate);
+void jtag_add_plain_dr_scan(int num_fields,
+		const struct scan_field* fields, tap_state_t endstate);
 
 /**
  * Defines the type of data passed to the jtag_callback_t interface.
@@ -427,24 +417,12 @@ typedef intptr_t jtag_callback_data_t;
 typedef void (*jtag_callback1_t)(jtag_callback_data_t data0);
 
 /// A simpler version of jtag_add_callback4().
-extern void jtag_add_callback(jtag_callback1_t, jtag_callback_data_t data0);
-
-
-
-/**
- * Defines the interface of the JTAG callback mechanism.
- *
- * @param in the pointer to the data clocked in
- * @param data1 An integer big enough to use as an @c int or a pointer.
- * @param data2 An integer big enough to use as an @c int or a pointer.
- * @param data3 An integer big enough to use as an @c int or a pointer.
- * @returns an error code
- */
-typedef int (*jtag_callback_t)(jtag_callback_data_t data0, jtag_callback_data_t data1, jtag_callback_data_t data2, jtag_callback_data_t data3);
+void jtag_add_callback(jtag_callback1_t, jtag_callback_data_t data0);
 
 
 /**
- * This callback can be executed immediately the queue has been flushed.
+ * Defines the interface of the JTAG callback mechanism.  Such
+ * callbacks can be executed once the queue has been flushed.
  *
  * The JTAG queue can be executed synchronously or asynchronously.
  * Typically for USB, the queue is executed asynchronously.  For
@@ -458,29 +436,27 @@ typedef int (*jtag_callback_t)(jtag_callback_data_t data0, jtag_callback_data_t
  *
  * If the execution of the queue fails before the callbacks, then --
  * depending on driver implementation -- the callbacks may or may not be
- * invoked.  @todo Can we make this behavior consistent?
+ * invoked.
  *
- * The strange name is due to C's lack of overloading using function
- * arguments.
+ * @todo Make that behavior consistent.
  *
- * @param f The callback function to add.
  * @param data0 Typically used to point to the data to operate on.
  * Frequently this will be the data clocked in during a shift operation.
  * @param data1 An integer big enough to use as an @c int or a pointer.
  * @param data2 An integer big enough to use as an @c int or a pointer.
  * @param data3 An integer big enough to use as an @c int or a pointer.
- *
+ * @returns an error code
  */
-extern void jtag_add_callback4(jtag_callback_t f, jtag_callback_data_t data0,
-		jtag_callback_data_t data1, jtag_callback_data_t data2,
-		jtag_callback_data_t data3);
-
+typedef int (*jtag_callback_t)(jtag_callback_data_t data0,
+				jtag_callback_data_t data1,
+				jtag_callback_data_t data2,
+				jtag_callback_data_t data3);
 
 /**
  * Run a TAP_RESET reset where the end state is TAP_RESET,
  * regardless of the start state.
  */
-extern void jtag_add_tlr(void);
+void jtag_add_tlr(void);
 
 /**
  * Application code *must* assume that interfaces will
@@ -530,7 +506,7 @@ extern void jtag_add_tlr(void);
  *   - ERROR_JTAG_TRANSITION_INVALID -- The path includes invalid
  *     state transitions.
  */
-extern void jtag_add_pathmove(int num_states, const tap_state_t* path);
+void jtag_add_pathmove(int num_states, const tap_state_t* path);
 
 /**
  * jtag_add_statemove() moves from the current state to @a goal_state.
@@ -541,7 +517,7 @@ extern void jtag_add_pathmove(int num_states, const tap_state_t* path);
  * Moves from the current state to the goal \a state.
  * Both states must be stable.
  */
-extern int jtag_add_statemove(tap_state_t goal_state);
+int jtag_add_statemove(tap_state_t goal_state);
 
 /**
  * Goes to TAP_IDLE (if we're not already there), cycle
@@ -549,11 +525,11 @@ extern int jtag_add_statemove(tap_state_t goal_state);
  * to @a endstate (unless it is also TAP_IDLE).
  *
  * @param num_cycles Number of cycles in TAP_IDLE state.  This argument
- * 	may be 0, in which case this routine will navigate to @a endstate
- * 	via TAP_IDLE.
+ *	may be 0, in which case this routine will navigate to @a endstate
+ *	via TAP_IDLE.
  * @param endstate The final state.
  */
-extern void jtag_add_runtest(int num_cycles, tap_state_t endstate);
+void jtag_add_runtest(int num_cycles, tap_state_t endstate);
 
 /**
  * A reset of the TAP state machine can be requested.
@@ -578,7 +554,7 @@ extern void jtag_add_runtest(int num_cycles, tap_state_t endstate);
  * are undefined, but when srst_pulls_trst or vice versa,
  * then trst & srst *must* be asserted together.
  */
-extern void jtag_add_reset(int req_tlr_or_trst, int srst);
+void jtag_add_reset(int req_tlr_or_trst, int srst);
 
 
 /**
@@ -587,23 +563,22 @@ extern void jtag_add_reset(int req_tlr_or_trst, int srst);
  * Set a global variable to \a state if \a state != TAP_INVALID.
  *
  * Return the value of the global variable.
- *
- **/
-extern tap_state_t jtag_set_end_state(tap_state_t state);
+ */
+tap_state_t jtag_set_end_state(tap_state_t state);
+
 /**
  * Function jtag_get_end_state
  *
  * Return the value of the global variable for end state
- *
- **/
-extern tap_state_t jtag_get_end_state(void);
-extern void jtag_add_sleep(uint32_t us);
+ */
+tap_state_t jtag_get_end_state(void);
 
+void jtag_add_sleep(uint32_t us);
 
 /**
- * Function jtag_add_stable_clocks
+ * Function jtag_add_clocks
  * first checks that the state in which the clocks are to be issued is
- * stable, then queues up clock_count clocks for transmission.
+ * stable, then queues up num_cycles clocks for transmission.
  */
 void jtag_add_clocks(int num_cycles);
 
@@ -628,10 +603,10 @@ void jtag_add_clocks(int num_cycles);
  * jtag_add_xxx() commands can either be executed immediately or
  * at some time between the jtag_add_xxx() fn call and jtag_execute_queue().
  */
-extern int jtag_execute_queue(void);
+int jtag_execute_queue(void);
 
 /// same as jtag_execute_queue() but does not clear the error flag
-extern void jtag_execute_queue_noclear(void);
+void jtag_execute_queue_noclear(void);
 
 /// @returns the number of times the scan queue has been flushed
 int jtag_get_flush_queue_count(void);
@@ -641,8 +616,8 @@ void jtag_notify_event(enum jtag_event);
 
 
 /* can be implemented by hw + sw */
-extern int jtag_power_dropout(int* dropout);
-extern int jtag_srst_asserted(int* srst_asserted);
+int jtag_power_dropout(int* dropout);
+int jtag_srst_asserted(int* srst_asserted);
 
 /* JTAG support functions */
 
@@ -653,9 +628,9 @@ extern int jtag_srst_asserted(int* srst_asserted);
  * @param mask Pointer to scan mask; may be NULL.
  * @returns Nothing, but calls jtag_set_error() on any error.
  */
-extern void jtag_check_value_mask(scan_field_t *field, uint8_t *value, uint8_t *mask);
+void jtag_check_value_mask(struct scan_field *field, uint8_t *value, uint8_t *mask);
 
-extern void jtag_sleep(uint32_t us);
+void jtag_sleep(uint32_t us);
 
 /*
  * The JTAG subsystem defines a number of error codes,
@@ -698,9 +673,6 @@ extern void jtag_sleep(uint32_t us);
  * There is no jtag_add_dr_outin() version of this fn that also allows
  * clocking data back in. Patches gladly accepted!
  */
-extern void jtag_add_dr_out(jtag_tap_t* tap,
-		int num_fields, const int* num_bits, const uint32_t* value,
-		tap_state_t end_state);
 
 
 /**
@@ -735,4 +707,9 @@ bool jtag_poll_get_enabled(void);
  */
 void jtag_poll_set_enabled(bool value);
 
+
+/* The minidriver may have inline versions of some of the low
+ * level APIs that are used in inner loops. */
+#include <jtag/minidriver.h>
+
 #endif /* JTAG_H */
