@@ -20,19 +20,17 @@
 #include "config.h"
 #endif
 
-#include "embeddedice.h"
-#include "minidriver.h"
-#include "interface.h"
+#include <jtag/jtag.h>
+#include <target/embeddedice.h>
+#include <jtag/minidriver.h>
+#include <jtag/interface.h>
 
-
-
-
-jtag_interface_t minidummy_interface =
+struct jtag_interface minidummy_interface =
 {
 	.name = "minidummy",
 	.execute_queue = NULL,
 	.speed = NULL,
-	.register_commands = NULL,
+	.commands = NULL,
 	.init = NULL,
 	.quit = NULL,
 	.khz = NULL,
@@ -41,11 +39,6 @@ jtag_interface_t minidummy_interface =
 	.srst_asserted = NULL,
 };
 
-
-
-
-
-
 int interface_jtag_execute_queue(void)
 {
 	/* synchronously do the operation here */
@@ -53,13 +46,7 @@ int interface_jtag_execute_queue(void)
 	return ERROR_OK;
 }
 
-
-
-
-
-extern int jtag_check_value(uint8_t *captured, void *priv);
-
-int interface_jtag_add_ir_scan(int num_fields, const scan_field_t *fields, tap_state_t state)
+int interface_jtag_add_ir_scan(int num_fields, const struct scan_field *fields, tap_state_t state)
 {
 	/* synchronously do the operation here */
 
@@ -67,34 +54,27 @@ int interface_jtag_add_ir_scan(int num_fields, const scan_field_t *fields, tap_s
 
 }
 
-
-
-
-
-int interface_jtag_add_plain_ir_scan(int num_fields, const scan_field_t *fields, tap_state_t state)
+int interface_jtag_add_plain_ir_scan(int num_fields, const struct scan_field *fields, tap_state_t state)
 {
 	/* synchronously do the operation here */
 
 	return ERROR_OK;
 }
 
-/*extern jtag_command_t **jtag_get_last_command_p(void);*/
-
-int interface_jtag_add_dr_scan(int num_fields, const scan_field_t *fields, tap_state_t state)
+int interface_jtag_add_dr_scan(int num_fields, const struct scan_field *fields, tap_state_t state)
 {
 	/* synchronously do the operation here */
 
 	return ERROR_OK;
 }
 
-int interface_jtag_add_plain_dr_scan(int num_fields, const scan_field_t *fields, tap_state_t state)
+int interface_jtag_add_plain_dr_scan(int num_fields, const struct scan_field *fields, tap_state_t state)
 {
 	/* synchronously do the operation here */
 
 	return ERROR_OK;
 }
 
-
 int interface_jtag_add_tlr()
 {
 	/* synchronously do the operation here */
@@ -102,8 +82,6 @@ int interface_jtag_add_tlr()
 	return ERROR_OK;
 }
 
-
-
 int interface_jtag_add_reset(int req_trst, int req_srst)
 {
 	/* synchronously do the operation here */
@@ -111,7 +89,6 @@ int interface_jtag_add_reset(int req_trst, int req_srst)
 	return ERROR_OK;
 }
 
-
 int interface_jtag_add_runtest(int num_cycles, tap_state_t state)
 {
 	/* synchronously do the operation here */
@@ -170,9 +147,7 @@ int interface_jtag_add_pathmove(int num_states, const tap_state_t *path)
 	return ERROR_OK;
 }
 
-
-
-void embeddedice_write_dcc(jtag_tap_t *tap, int reg_addr, uint8_t *buffer, int little, int count)
+void embeddedice_write_dcc(struct jtag_tap *tap, int reg_addr, uint8_t *buffer, int little, int count)
 {
 	int i;
 	for (i = 0; i < count; i++)
@@ -181,4 +156,3 @@ void embeddedice_write_dcc(jtag_tap_t *tap, int reg_addr, uint8_t *buffer, int l
 		buffer += 4;
 	}
 }
-
