@@ -26,30 +26,18 @@
 #include "pld.h"
 
 
-static int virtex2_register_commands(struct command_context_s *cmd_ctx);
-static int virtex2_pld_device_command(struct command_context_s *cmd_ctx, char *cmd, char **args, int argc, struct pld_device_s *pld_device);
-static int virtex2_load(struct pld_device_s *pld_device, char *filename);
-
-pld_driver_t virtex2_pld =
-{
-	.name = "virtex2",
-	.register_commands = virtex2_register_commands,
-	.pld_device_command = virtex2_pld_device_command,
-	.load = virtex2_load,
-};
-
-static int virtex2_set_instr(jtag_tap_t *tap, uint32_t new_instr)
+static int virtex2_set_instr(struct jtag_tap *tap, uint32_t new_instr)
 {
 	if (tap == NULL)
 		return ERROR_FAIL;
 
 	if (buf_get_u32(tap->cur_instr, 0, tap->ir_length) != new_instr)
 	{
-		scan_field_t field;
+		struct scan_field field;
 
 		field.tap = tap;
 		field.num_bits = tap->ir_length;
-		field.out_value = calloc(CEIL(field.num_bits, 8), 1);
+		field.out_value = calloc(DIV_ROUND_UP(field.num_bits, 8), 1);
 		buf_set_u32(field.out_value, 0, field.num_bits, new_instr);
 		field.in_value = NULL;
 
@@ -61,11 +49,11 @@ static int virtex2_set_instr(jtag_tap_t *tap, uint32_t new_instr)
 	return ERROR_OK;
 }
 
-static int virtex2_send_32(struct pld_device_s *pld_device,
+static int virtex2_send_32(struct pld_device *pld_device,
 		int num_words, uint32_t *words)
 {
-	virtex2_pld_device_t *virtex2_info = pld_device->driver_priv;
-	scan_field_t scan_field;
+	struct virtex2_pld_device *virtex2_info = pld_device->driver_priv;
+	struct scan_field scan_field;
 	uint8_t *values;
 	int i;
 
@@ -94,11 +82,11 @@ static __inline__ void virtexflip32(jtag_callback_data_t arg)
 	*((uint32_t *)in) = flip_u32(le_to_h_u32(in), 32);
 }
 
-static int virtex2_receive_32(struct pld_device_s *pld_device,
+static int virtex2_receive_32(struct pld_device *pld_device,
 		int num_words, uint32_t *words)
 {
-	virtex2_pld_device_t *virtex2_info = pld_device->driver_priv;
-	scan_field_t scan_field;
+	struct virtex2_pld_device *virtex2_info = pld_device->driver_priv;
+	struct scan_field scan_field;
 
 	scan_field.tap = virtex2_info->tap;
 	scan_field.num_bits = 32;
@@ -121,7 +109,7 @@ static int virtex2_receive_32(struct pld_device_s *pld_device,
 	return ERROR_OK;
 }
 
-static int virtex2_read_stat(struct pld_device_s *pld_device, uint32_t *status)
+static int virtex2_read_stat(struct pld_device *pld_device, uint32_t *status)
 {
 	uint32_t data[5];
 
@@ -143,13 +131,13 @@ static int virtex2_read_stat(struct pld_device_s *pld_device, uint32_t *status)
 	return ERROR_OK;
 }
 
-static int virtex2_load(struct pld_device_s *pld_device, char *filename)
+static int virtex2_load(struct pld_device *pld_device, const char *filename)
 {
-	virtex2_pld_device_t *virtex2_info = pld_device->driver_priv;
-	xilinx_bit_file_t bit_file;
+	struct virtex2_pld_device *virtex2_info = pld_device->driver_priv;
+	struct xilinx_bit_file bit_file;
 	int retval;
 	unsigned int i;
-	scan_field_t field;
+	struct scan_field field;
 
 	field.tap = virtex2_info->tap;
 	field.in_value = NULL;
@@ -189,23 +177,24 @@ static int virtex2_load(struct pld_device_s *pld_device, char *filename)
 	return ERROR_OK;
 }
 
-static int virtex2_handle_read_stat_command(struct command_context_s *cmd_ctx,
-		char *cmd, char **args, int argc)
+COMMAND_HANDLER(virtex2_handle_read_stat_command)
 {
-	pld_device_t *device;
-	virtex2_pld_device_t *virtex2_info;
+	struct pld_device *device;
+	struct virtex2_pld_device *virtex2_info;
 	uint32_t status;
 
-	if (argc < 1)
+	if (CMD_ARGC < 1)
 	{
-		command_print(cmd_ctx, "usage: virtex2 read_stat <num>");
+		command_print(CMD_CTX, "usage: virtex2 read_stat <num>");
 		return ERROR_OK;
 	}
 
-	device = get_pld_device_by_num(strtoul(args[0], NULL, 0));
+	unsigned dev_id;
+	COMMAND_PARSE_NUMBER(uint, CMD_ARGV[0], dev_id);
+	device = get_pld_device_by_num(dev_id);
 	if (!device)
 	{
-		command_print(cmd_ctx, "pld device '#%s' is out of bounds", args[0]);
+		command_print(CMD_CTX, "pld device '#%s' is out of bounds", CMD_ARGV[0]);
 		return ERROR_OK;
 	}
 
@@ -213,43 +202,60 @@ static int virtex2_handle_read_stat_command(struct command_context_s *cmd_ctx,
 
 	virtex2_read_stat(device, &status);
 
-	command_print(cmd_ctx, "virtex2 status register: 0x%8.8" PRIx32 "", status);
+	command_print(CMD_CTX, "virtex2 status register: 0x%8.8" PRIx32 "", status);
 
 	return ERROR_OK;
 }
 
-static int virtex2_register_commands(struct command_context_s *cmd_ctx)
+PLD_DEVICE_COMMAND_HANDLER(virtex2_pld_device_command)
 {
-	command_t *virtex2_cmd = register_command(cmd_ctx, NULL, "virtex2", NULL, COMMAND_ANY, "virtex2 specific commands");
+	struct jtag_tap *tap;
 
-	register_command(cmd_ctx, virtex2_cmd, "read_stat", virtex2_handle_read_stat_command, COMMAND_EXEC,
-					 "read Virtex-II status register");
+	struct virtex2_pld_device *virtex2_info;
 
-	return ERROR_OK;
-}
-
-static int virtex2_pld_device_command(struct command_context_s *cmd_ctx,
-		char *cmd, char **args, int argc, struct pld_device_s *pld_device)
-{
-	jtag_tap_t *tap;
-
-	virtex2_pld_device_t *virtex2_info;
-
-	if (argc < 2)
+	if (CMD_ARGC < 2)
 	{
 		LOG_WARNING("incomplete pld device 'virtex2' configuration");
 		return ERROR_PLD_DEVICE_INVALID;
 	}
 
-	tap = jtag_tap_by_string(args[1]);
+	tap = jtag_tap_by_string(CMD_ARGV[1]);
 	if (tap == NULL) {
-		command_print(cmd_ctx, "Tap: %s does not exist", args[1]);
+		command_print(CMD_CTX, "Tap: %s does not exist", CMD_ARGV[1]);
 		return ERROR_OK;
 	}
 
-	virtex2_info = malloc(sizeof(virtex2_pld_device_t));
-	pld_device->driver_priv = virtex2_info;
+	virtex2_info = malloc(sizeof(struct virtex2_pld_device));
 	virtex2_info->tap = tap;
 
+	pld->driver_priv = virtex2_info;
+
 	return ERROR_OK;
 }
+
+static const struct command_registration virtex2_exec_command_handlers[] = {
+	{
+		.name = "read_stat",
+		.mode = COMMAND_EXEC,
+		.handler = virtex2_handle_read_stat_command,
+		.help = "read status register",
+		.usage = "pld_num",
+	},
+	COMMAND_REGISTRATION_DONE
+};
+static const struct command_registration virtex2_command_handler[] = {
+	{
+		.name = "virtex2",
+		.mode = COMMAND_ANY,
+		.help = "Virtex-II specific commands",
+		.chain = virtex2_exec_command_handlers,
+	},
+	COMMAND_REGISTRATION_DONE
+};
+
+struct pld_driver virtex2_pld = {
+		.name = "virtex2",
+		.commands = virtex2_command_handler,
+		.pld_device_command = &virtex2_pld_device_command,
+		.load = &virtex2_load,
+	};
