@@ -2,7 +2,7 @@
  *   Copyright (C) 2005 by Dominic Rath                                    *
  *   Dominic.Rath@gmx.de                                                   *
  *                                                                         *
- *   Copyright (C) 2007,2008 Øyvind Harboe                                 *
+ *   Copyright (C) 2007-2009 Øyvind Harboe                                 *
  *   oyvind.harboe@zylin.com                                               *
  *                                                                         *
  *   Copyright (C) 2008 by Spencer Oliver                                  *
@@ -26,36 +26,18 @@
 #ifndef GDB_SERVER_H
 #define GDB_SERVER_H
 
-#include "target.h"
-
-struct image_s;
+struct image;
+#include <target/target.h>
 
 #define GDB_BUFFER_SIZE	16384
 
-typedef struct gdb_connection_s
+struct gdb_service
 {
-	char buffer[GDB_BUFFER_SIZE];
-	char *buf_p;
-	int buf_cnt;
-	int ctrl_c;
-	enum target_state frontend_state;
-	struct image_s *vflash_image;
-	int closed;
-	int busy;
-	int noack_mode;
-	bool sync; 	/* set flag to true if you want the next stepi to return immediately.
-	               allowing GDB to pick up a fresh set of register values from the target
-	               without modifying the target state. */
+	struct target *target;
+};
 
-} gdb_connection_t;
-
-typedef struct gdb_service_s
-{
-	struct target_s *target;
-} gdb_service_t;
-
-extern int gdb_init(void);
-extern int gdb_register_commands(command_context_t *command_context);
+int gdb_target_add_all(struct target *target);
+int gdb_register_commands(struct command_context *command_context);
 
 #define ERROR_GDB_BUFFER_TOO_SMALL (-800)
 #define ERROR_GDB_TIMEOUT (-801)
