@@ -26,7 +26,7 @@
 #endif
 
 #include "telnet_server.h"
-#include "target.h"
+#include <target/target.h>
 
 #include <microhttpd.h>
 #include <pthread.h>
@@ -109,7 +109,7 @@ static int httpd_Jim_Command_writeform(Jim_Interp *interp, int argc,
 {
 	if (argc != 3)
 	{
-		Jim_WrongNumArgs(interp, 1, argv, "method ?args ...?");
+		Jim_WrongNumArgs(interp, 1, argv, "method ?CMD_ARGV ...?");
 		return JIM_ERR;
 	}
 	char *name = (char*) Jim_GetString(argv[1], NULL);
@@ -118,12 +118,12 @@ static int httpd_Jim_Command_writeform(Jim_Interp *interp, int argc,
 	// Find length
 	const char *data;
 	int actual;
-
 	int retcode;
-
-	const char *script = alloc_printf("set dummy_val $httppostdata(%s); set dummy_val",
+	const char *script = alloc_printf(
+			"set dummy_val $httppostdata(%s); set dummy_val",
 			name);
-	retcode = Jim_Eval_Named(interp, script, "httpd.c", __LINE__);
+
+	retcode = Jim_Eval_Named(interp, script, __FILE__, __LINE__);
 	free((void *) script);
 	if (retcode != JIM_OK)
 		return retcode;
@@ -154,32 +154,31 @@ httpd_Jim_Command_formfetch(Jim_Interp *interp,
                                    int argc,
                                    Jim_Obj *const *argv)
 {
-    if (argc != 2)
-    {
-        Jim_WrongNumArgs(interp, 1, argv, "method ?args ...?");
-        return JIM_ERR;
-    }
-    char *name = (char*)Jim_GetString(argv[1], NULL);
+	if (argc != 2)
+	{
+		Jim_WrongNumArgs(interp, 1, argv, "method ?CMD_ARGV ...?");
+		return JIM_ERR;
+	}
 
+	char *name = (char*)Jim_GetString(argv[1], NULL);
+	const char *script = alloc_printf(
+		"set dummy_val $httppostdata(%s); set dummy_val",
+		name);
+	int retcode = Jim_Eval_Named(interp, script, __FILE__, __LINE__);
 
-    const char *script = alloc_printf("set dummy_val $httppostdata(%s); set dummy_val",
-    			name);
-    	int retcode = Jim_Eval_Named(interp, script, "httpd.c", __LINE__);
-    	free((void *) script);
-    	if (retcode != JIM_OK)
-    	{
-    	    Jim_SetResult(interp, Jim_NewEmptyStringObj(interp));
-    	} else
-    	{
-    	    Jim_SetResult(interp, Jim_GetResult(interp));
-    	}
+	free((void *) script);
+	if (retcode != JIM_OK)
+		Jim_SetResult(interp, Jim_NewEmptyStringObj(interp));
+	else
+		Jim_SetResult(interp, Jim_GetResult(interp));
 
-    return JIM_OK;
+	return JIM_OK;
 }
 
 struct httpd_request
 {
 	int post;
+	Jim_Interp *interp;
 	struct MHD_PostProcessor *postprocessor;
 
 	//Jim_Obj *dict;
@@ -207,8 +206,9 @@ static void request_completed(void *cls, struct MHD_Connection *connection,
 	*con_cls = NULL;
 }
 
-/* append to said key in dictonary */
-static void append_key(struct httpd_request *r, const char *key,
+/* append to said key in dictionary */
+static void append_key(Jim_Interp *interp,
+		struct httpd_request *r, const char *key,
 		const char *data, size_t off, size_t size)
 {
 	Jim_Obj *keyObj = Jim_NewStringObj(interp, key, -1);
@@ -259,7 +259,7 @@ static int iterate_post(void *con_cls, enum MHD_ValueKind kind,
 {
 	struct httpd_request *r = (struct httpd_request*) con_cls;
 
-	append_key(r, key, data, off, size);
+	append_key(r->interp, r, key, data, off, size);
 
 	return MHD_YES;
 }
@@ -268,12 +268,13 @@ static int record_arg(void *cls, enum MHD_ValueKind kind, const char *key,
 		const char *value)
 {
 	struct httpd_request *r = (struct httpd_request*) cls;
-	append_key(r, key, value, 0, strlen(value));
+	append_key(r->interp, r, key, value, 0, strlen(value));
 	return MHD_YES;
 }
 
 
-static int handle_request(struct MHD_Connection * connection, const char * url)
+static int handle_request(Jim_Interp *interp,
+		struct MHD_Connection * connection, const char * url)
 {
 	struct MHD_Response * response;
 
@@ -288,10 +289,10 @@ static int handle_request(struct MHD_Connection * connection, const char * url)
 
 		const char *script = alloc_printf(
 				"global httpdata; source {%s}; set httpdata", url);
-		retcode = Jim_Eval_Named(interp, script, "httpd.c", __LINE__);
+		retcode = Jim_Eval_Named(interp, script, __FILE__, __LINE__);
 		free((void *) script);
 
-		if (retcode == JIM_ERR)
+		if (retcode != JIM_OK)
 		{
 			printf("Tcl failed\n");
 			const char *t = httpd_exec_cgi_tcl_error(interp);
@@ -341,7 +342,10 @@ static int handle_request(struct MHD_Connection * connection, const char * url)
 		LOG_DEBUG("Serving %s length=%zu", url, len);
 		/* serve file directly */
 		response = MHD_create_response_from_data(len, data, MHD_YES, MHD_NO);
+		/* Should we expose mimetype via tcl here or just let the browser
+		   guess?
 		MHD_add_response_header(response, "Content-Type", "image/png");
+		*/
 
 		ret = MHD_queue_response(connection, MHD_HTTP_OK, response);
 		MHD_destroy_response(response);
@@ -353,8 +357,9 @@ static int handle_request(struct MHD_Connection * connection, const char * url)
 
 static int ahc_echo_inner(void * cls, struct MHD_Connection * connection,
 		const char * url, const char * method, const char * version,
-		const char * upload_data, unsigned int * upload_data_size, void ** ptr)
+		const char * upload_data, size_t * upload_data_size, void ** ptr)
 {
+	Jim_Interp *interp = (Jim_Interp *)cls;
 	int post = 0;
 
 	if (0 == strcmp(method, "POST"))
@@ -381,11 +386,11 @@ static int ahc_echo_inner(void * cls, struct MHD_Connection * connection,
 		memset(*ptr, 0, sizeof(struct httpd_request));
 
 		r = (struct httpd_request *) *ptr;
-
+		r->interp = interp;
 		r->post = post;
 		Jim_SetVariableStr(interp, "httppostdata", Jim_NewDictObj(interp, NULL, 0));
 
-		/* fill in url query strings in dictonary */
+		/* fill in url query strings in dictionary */
 		MHD_get_connection_values(connection, MHD_GET_ARGUMENT_KIND,
 				record_arg, r);
 
@@ -434,7 +439,7 @@ static int ahc_echo_inner(void * cls, struct MHD_Connection * connection,
 		url="index.tcl";
 
 	const char *file_name = alloc_printf("%s/%s", httpd_dir, url);
-	int result = handle_request(connection, file_name);
+	int result = handle_request(interp, connection, file_name);
 	free((void *)file_name);
 	return result;
 }
@@ -442,7 +447,7 @@ static int ahc_echo_inner(void * cls, struct MHD_Connection * connection,
 
 static int ahc_echo(void * cls, struct MHD_Connection * connection,
 		const char * url, const char * method, const char * version,
-		const char * upload_data, unsigned int * upload_data_size, void ** ptr)
+		const char * upload_data, size_t * upload_data_size, void ** ptr)
 {
 	int result;
 
@@ -457,7 +462,25 @@ static int ahc_echo(void * cls, struct MHD_Connection * connection,
 
 static struct MHD_Daemon * d;
 
-int httpd_start(void)
+static const struct command_registration httpd_command_handlers[] = {
+	{
+		.name = "formfetch",
+		.jim_handler = httpd_Jim_Command_formfetch,
+		.mode = COMMAND_EXEC,
+		.usage = "parameter_name",
+		.help = "Reads a posted form value.",
+	},
+	{
+		.name = "writeform",
+		.jim_handler = httpd_Jim_Command_writeform,
+		.mode = COMMAND_EXEC,
+		.usage = "parameter_name filename",
+		.help = "Writes a form value to a file.",
+	},
+	COMMAND_REGISTRATION_DONE
+};
+
+int httpd_start(struct command_context *cmd_ctx)
 {
 	pthread_mutexattr_t attr;
 	pthread_mutexattr_init(&attr);
@@ -466,26 +489,13 @@ int httpd_start(void)
 	int port = 8888;
 	LOG_USER("Launching httpd server on port %d", port);
 	d = MHD_start_daemon(MHD_USE_SELECT_INTERNALLY, port, NULL, NULL,
-			&ahc_echo, NULL, /* could be data for handler, but we only have a single handler, use global variables instead */
+			&ahc_echo, cmd_ctx->interp,
 			MHD_OPTION_NOTIFY_COMPLETED, request_completed, NULL, /* Closure... what's that??? */
 			MHD_OPTION_END);
 	if (d == NULL)
 		return ERROR_FAIL;
 
-	Jim_CreateCommand(interp,
-						  "formfetch",
-						  httpd_Jim_Command_formfetch,
-						  NULL,
-						  NULL);
-
-    Jim_CreateCommand(interp,
-                      "writeform",
-                      httpd_Jim_Command_writeform,
-                      NULL,
-                      NULL);
-
-
-	return ERROR_OK;
+	return register_commands(cmd_ctx, NULL, httpd_command_handlers);
 }
 
 void httpd_stop(void)
