@@ -26,7 +26,7 @@
 #ifndef SERVER_H
 #define SERVER_H
 
-#include "log.h"
+#include <helper/log.h>
 
 #ifdef HAVE_NETINET_IN_H
 #include <netinet/in.h>
@@ -38,22 +38,22 @@ enum connection_type
 	CONNECTION_PIPE
 };
 
-typedef struct connection_s
+struct connection
 {
 	int fd;
 	struct sockaddr_in sin;
-	command_context_t *cmd_ctx;
-	struct service_s *service;
+	struct command_context *cmd_ctx;
+	struct service *service;
 	int input_pending;
 	void *priv;
-	struct connection_s *next;
-} connection_t;
+	struct connection *next;
+};
 
-typedef int (*new_connection_handler_t)(connection_t *connection);
-typedef int (*input_handler_t)(connection_t *connection);
-typedef int (*connection_closed_handler_t)(connection_t *connection);
+typedef int (*new_connection_handler_t)(struct connection *connection);
+typedef int (*input_handler_t)(struct connection *connection);
+typedef int (*connection_closed_handler_t)(struct connection *connection);
 
-typedef struct service_s
+struct service
 {
 	char *name;
 	enum connection_type type;
@@ -61,19 +61,46 @@ typedef struct service_s
 	int fd;
 	struct sockaddr_in sin;
 	int max_connections;
-	connection_t *connections;
+	struct connection *connections;
 	new_connection_handler_t new_connection;
 	input_handler_t input;
 	connection_closed_handler_t connection_closed;
 	void *priv;
-	struct service_s *next;
-} service_t;
+	struct service *next;
+};
 
-extern int add_service(char *name, enum connection_type type, unsigned short port, int max_connections, new_connection_handler_t new_connection_handler, input_handler_t input_handler, connection_closed_handler_t connection_closed_handler, void *priv);
-extern int server_init(void);
-extern int server_quit(void);
-extern int server_loop(command_context_t *command_context);
-extern int server_register_commands(command_context_t *context);
+int add_service(char *name, enum connection_type type, unsigned short port,
+		int max_connections, new_connection_handler_t new_connection_handler,
+		input_handler_t in_handler, connection_closed_handler_t close_handler,
+		void *priv);
+
+int server_preinit(void);
+int server_init(struct command_context *cmd_ctx);
+int server_quit(void);
+
+int server_loop(struct command_context *command_context);
+
+int server_register_commands(struct command_context *context);
+
+/**
+ * Used by server_loop(), defined in server_stubs.c, httpd.c, or ecosboard.c
+ */
+void openocd_sleep_prelude(void);
+/**
+ * Used by server_loop(), defined in server_stubs.c, httpd.c, or ecosboard.c
+ */
+void openocd_sleep_postlude(void);
+
+/**
+ * Defines an extended command handler function declaration to enable
+ * access to (and manipulation of) the server port number.
+ * Call server_port like a normal COMMAND_HANDLER with an extra @a out parameter
+ * to receive the specified port number.
+ */
+#define SERVER_PORT_COMMAND() \
+		COMMAND_HELPER(server_port_command, unsigned short *out)
+
+SERVER_PORT_COMMAND();
 
 extern int server_use_pipes;
 
