@@ -28,13 +28,10 @@
 #endif
 
 #include "telnet_server.h"
-#include "target_request.h"
+#include <target/target_request.h>
 
 static unsigned short telnet_port = 4444;
 
-int handle_exit_command(struct command_context_s *cmd_ctx, char *cmd, char **args, int argc);
-int handle_telnet_port_command(struct command_context_s *cmd_ctx, char *cmd, char **args, int argc);
-
 static char *negotiate =
 		"\xFF\xFB\x03"		/* IAC WILL Suppress Go Ahead */
 		"\xFF\xFB\x01"		/* IAC WILL Echo */
@@ -47,9 +44,9 @@ static char *negotiate =
  * we write to it, we will fail. Subsequent write operations will
  * succeed. Shudder!
  */
-int telnet_write(connection_t *connection, const void *data, int len)
+int telnet_write(struct connection *connection, const void *data, int len)
 {
-	telnet_connection_t *t_con = connection->priv;
+	struct telnet_connection *t_con = connection->priv;
 	if (t_con->closed)
 		return ERROR_SERVER_REMOTE_CLOSED;
 
@@ -61,15 +58,14 @@ int telnet_write(connection_t *connection, const void *data, int len)
 	return ERROR_SERVER_REMOTE_CLOSED;
 }
 
-int telnet_prompt(connection_t *connection)
+int telnet_prompt(struct connection *connection)
 {
-	telnet_connection_t *t_con = connection->priv;
+	struct telnet_connection *t_con = connection->priv;
 
-	telnet_write(connection, "\r", 1); /* the prompt is always placed at the line beginning */
 	return telnet_write(connection, t_con->prompt, strlen(t_con->prompt));
 }
 
-int telnet_outputline(connection_t *connection, const char *line)
+int telnet_outputline(struct connection *connection, const char *line)
 {
 	int len;
 
@@ -97,18 +93,18 @@ int telnet_outputline(connection_t *connection, const char *line)
 	return ERROR_OK;
 }
 
-int telnet_output(struct command_context_s *cmd_ctx, const char* line)
+int telnet_output(struct command_context *cmd_ctx, const char* line)
 {
-	connection_t *connection = cmd_ctx->output_handler_priv;
+	struct connection *connection = cmd_ctx->output_handler_priv;
 
 	return telnet_outputline(connection, line);
 }
 
-void telnet_log_callback(void *priv, const char *file, int line,
+void telnet_log_callback(void *priv, const char *file, unsigned line,
 		const char *function, const char *string)
 {
-	connection_t *connection = priv;
-	telnet_connection_t *t_con = connection->priv;
+	struct connection *connection = priv;
+	struct telnet_connection *t_con = connection->priv;
 	int i;
 
 	/* if there is no prompt, simply output the message */
@@ -119,10 +115,12 @@ void telnet_log_callback(void *priv, const char *file, int line,
 	}
 
 	/* clear the command line */
-	telnet_write(connection, "\r", 1);
+	for (i = strlen(t_con->prompt) + t_con->line_size; i > 0; i -= 16)
+		telnet_write(connection, "\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b", i > 16 ? 16 : i);
 	for (i = strlen(t_con->prompt) + t_con->line_size; i > 0; i -= 16)
 		telnet_write(connection, "                ", i > 16 ? 16 : i);
-	telnet_write(connection, "\r", 1);
+	for (i = strlen(t_con->prompt) + t_con->line_size; i > 0; i -= 16)
+		telnet_write(connection, "\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b", i > 16 ? 16 : i);
 
 	/* output the message */
 	telnet_outputline(connection, string);
@@ -134,10 +132,10 @@ void telnet_log_callback(void *priv, const char *file, int line,
 		telnet_write(connection, "\b", 1);
 }
 
-int telnet_new_connection(connection_t *connection)
+int telnet_new_connection(struct connection *connection)
 {
-	telnet_connection_t *telnet_connection = malloc(sizeof(telnet_connection_t));
-	telnet_service_t *telnet_service = connection->service->priv;
+	struct telnet_connection *telnet_connection = malloc(sizeof(struct telnet_connection));
+	struct telnet_service *telnet_service = connection->service->priv;
 	int i;
 
 	connection->priv = telnet_connection;
@@ -163,6 +161,7 @@ int telnet_new_connection(connection_t *connection)
 		telnet_write(connection, "\r\n", 2);
 	}
 
+	telnet_write(connection, "\r", 1); /* the prompt is always placed at the line beginning */
 	telnet_prompt(connection);
 
 	/* initialize history */
@@ -178,7 +177,7 @@ int telnet_new_connection(connection_t *connection)
 	return ERROR_OK;
 }
 
-void telnet_clear_line(connection_t *connection, telnet_connection_t *t_con)
+void telnet_clear_line(struct connection *connection, struct telnet_connection *t_con)
 {
 	/* move to end of line */
 	if (t_con->line_cursor < t_con->line_size)
@@ -195,13 +194,13 @@ void telnet_clear_line(connection_t *connection, telnet_connection_t *t_con)
 	t_con->line_cursor = 0;
 }
 
-int telnet_input(connection_t *connection)
+int telnet_input(struct connection *connection)
 {
 	int bytes_read;
-	char buffer[TELNET_BUFFER_SIZE];
-	char *buf_p;
-	telnet_connection_t *t_con = connection->priv;
-	command_context_t *command_context = connection->cmd_ctx;
+	unsigned char buffer[TELNET_BUFFER_SIZE];
+	unsigned char *buf_p;
+	struct telnet_connection *t_con = connection->priv;
+	struct command_context *command_context = connection->cmd_ctx;
 
 	bytes_read = read_socket(connection->fd, buffer, TELNET_BUFFER_SIZE);
 
@@ -219,7 +218,7 @@ int telnet_input(connection_t *connection)
 		switch (t_con->state)
 		{
 			case TELNET_STATE_DATA:
-				if (*buf_p == '\xff')
+				if (*buf_p == 0xff)
 				{
 					t_con->state = TELNET_STATE_IAC;
 				}
@@ -334,6 +333,7 @@ int telnet_input(connection_t *connection)
 							if (retval == ERROR_COMMAND_CLOSE_CONNECTION)
 								return ERROR_SERVER_REMOTE_CLOSED;
 
+							telnet_write(connection, "\r", 1); /* the prompt is always placed at the line beginning */
 							retval = telnet_prompt(connection);
 							if (retval == ERROR_SERVER_REMOTE_CLOSED)
 								return ERROR_SERVER_REMOTE_CLOSED;
@@ -398,16 +398,16 @@ int telnet_input(connection_t *connection)
 			case TELNET_STATE_IAC:
 				switch (*buf_p)
 				{
-					case '\xfe':
+					case 0xfe:
 						t_con->state = TELNET_STATE_DONT;
 						break;
-					case '\xfd':
+					case 0xfd:
 						t_con->state = TELNET_STATE_DO;
 						break;
-					case '\xfc':
+					case 0xfc:
 						t_con->state = TELNET_STATE_WONT;
 						break;
-					case '\xfb':
+					case 0xfb:
 						t_con->state = TELNET_STATE_WILL;
 						break;
 				}
@@ -540,9 +540,9 @@ int telnet_input(connection_t *connection)
 	return ERROR_OK;
 }
 
-int telnet_connection_closed(connection_t *connection)
+int telnet_connection_closed(struct connection *connection)
 {
-	telnet_connection_t *t_con = connection->priv;
+	struct telnet_connection *t_con = connection->priv;
 	int i;
 
 	log_remove_callback(telnet_log_callback, connection);
@@ -578,9 +578,9 @@ int telnet_connection_closed(connection_t *connection)
 	return ERROR_OK;
 }
 
-int telnet_set_prompt(connection_t *connection, char *prompt)
+int telnet_set_prompt(struct connection *connection, char *prompt)
 {
-	telnet_connection_t *t_con = connection->priv;
+	struct telnet_connection *t_con = connection->priv;
 
 	if (t_con->prompt != NULL)
 		free(t_con->prompt);
@@ -592,7 +592,7 @@ int telnet_set_prompt(connection_t *connection, char *prompt)
 
 int telnet_init(char *banner)
 {
-	telnet_service_t *telnet_service = malloc(sizeof(telnet_service_t));
+	struct telnet_service *telnet_service = malloc(sizeof(struct telnet_service));
 
 	if (telnet_port == 0)
 	{
@@ -608,32 +608,37 @@ int telnet_init(char *banner)
 	return ERROR_OK;
 }
 
-int telnet_register_commands(command_context_t *command_context)
-{
-	register_command(command_context, NULL, "exit", handle_exit_command,
-					 COMMAND_EXEC, "exit telnet session");
-
-	register_command(command_context, NULL, "telnet_port", handle_telnet_port_command,
-					 COMMAND_ANY, "port on which to listen for incoming telnet connections");
-
-	return ERROR_OK;
-}
-
 /* daemon configuration command telnet_port */
-int handle_telnet_port_command(struct command_context_s *cmd_ctx, char *cmd, char **args, int argc)
+COMMAND_HANDLER(handle_telnet_port_command)
 {
-	if (argc == 0)
-	{
-		command_print(cmd_ctx, "%d", telnet_port);
-		return ERROR_OK;
-	}
-
-	telnet_port = strtoul(args[0], NULL, 0);
-
-	return ERROR_OK;
+	return CALL_COMMAND_HANDLER(server_port_command, &telnet_port);
 }
 
-int handle_exit_command(struct command_context_s *cmd_ctx, char *cmd, char **args, int argc)
+COMMAND_HANDLER(handle_exit_command)
 {
 	return ERROR_COMMAND_CLOSE_CONNECTION;
 }
+
+static const struct command_registration telnet_command_handlers[] = {
+	{
+		.name = "exit",
+		.handler = handle_exit_command,
+		.mode = COMMAND_EXEC,
+		.help = "exit telnet session",
+	},
+	{
+		.name = "telnet_port",
+		.handler = handle_telnet_port_command,
+		.mode = COMMAND_ANY,
+		.help = "Specify port on which to listen "
+			"for incoming telnet connections.  "
+			"No arguments reports telnet port; zero disables.",
+		.usage = "[port_num]",
+	},
+	COMMAND_REGISTRATION_DONE
+};
+
+int telnet_register_commands(struct command_context *cmd_ctx)
+{
+	return register_commands(cmd_ctx, NULL, telnet_command_handlers);
+}
