@@ -20,8 +20,6 @@
 #ifndef ALGORITHM_H
 #define ALGORITHM_H
 
-#include "types.h"
-
 enum param_direction
 {
 	PARAM_IN,
@@ -29,25 +27,28 @@ enum param_direction
 	PARAM_IN_OUT
 };
 
-typedef struct mem_param_s
+struct mem_param
 {
 	uint32_t address;
 	uint32_t size;
 	uint8_t *value;
 	enum param_direction direction;
-} mem_param_t;
+};
 
-typedef struct reg_param_s
+struct reg_param
 {
 	char *reg_name;
 	uint32_t size;
 	uint8_t *value;
 	enum param_direction direction;
-} reg_param_t;
+};
 
-extern void init_mem_param(mem_param_t *param, uint32_t address, uint32_t size, enum param_direction direction);
-extern void destroy_mem_param(mem_param_t *param);
-extern void init_reg_param(reg_param_t *param, char *reg_name, uint32_t size, enum param_direction direction);
-extern void destroy_reg_param(reg_param_t *param);
+void init_mem_param(struct mem_param *param,
+		uint32_t address, uint32_t size, enum param_direction dir);
+void destroy_mem_param(struct mem_param *param);
+
+void init_reg_param(struct reg_param *param,
+		char *reg_name, uint32_t size, enum param_direction dir);
+void destroy_reg_param(struct reg_param *param);
 
 #endif /* ALGORITHM_H */
