@@ -25,41 +25,53 @@
 
 #define	ARM920T_COMMON_MAGIC 0xa920a920
 
-typedef struct arm920t_common_s
+struct arm920t_common
 {
+	struct arm7_9_common arm7_9_common;
 	uint32_t common_magic;
-	armv4_5_mmu_common_t armv4_5_mmu;
-	arm9tdmi_common_t arm9tdmi_common;
+	struct armv4_5_mmu_common armv4_5_mmu;
 	uint32_t cp15_control_reg;
 	uint32_t d_fsr;
 	uint32_t i_fsr;
 	uint32_t d_far;
 	uint32_t i_far;
 	int preserve_cache;
-} arm920t_common_t;
+};
 
-typedef struct arm920t_cache_line_s
+static inline struct arm920t_common *
+target_to_arm920(struct target *target)
+{
+	return container_of(target->arch_info, struct arm920t_common,
+			arm7_9_common.armv4_5_common);
+}
+
+struct arm920t_cache_line
 {
 	uint32_t cam;
 	uint32_t data[8];
-} arm920t_cache_line_t;
+};
 
-typedef struct arm920t_tlb_entry_s
+struct arm920t_tlb_entry
 {
 	uint32_t cam;
 	uint32_t ram1;
 	uint32_t ram2;
-} arm920t_tlb_entry_t;
+};
 
-int arm920t_arch_state(struct target_s *target);
-int arm920t_soft_reset_halt(struct target_s *target);
-int arm920t_read_memory(struct target_s *target, uint32_t address, uint32_t size, uint32_t count, uint8_t *buffer);
-int arm920t_write_memory(struct target_s *target, uint32_t address, uint32_t size, uint32_t count, uint8_t *buffer);
-void arm920t_post_debug_entry(target_t *target);
-void arm920t_pre_restore_context(target_t *target);
-uint32_t arm920t_get_ttb(target_t *target);
-void arm920t_disable_mmu_caches(target_t *target, int mmu, int d_u_cache, int i_cache);
-void arm920t_enable_mmu_caches(target_t *target, int mmu, int d_u_cache, int i_cache);
-int arm920t_register_commands(struct command_context_s *cmd_ctx);
+int arm920t_arch_state(struct target *target);
+int arm920t_soft_reset_halt(struct target *target);
+int arm920t_read_memory(struct target *target,
+	uint32_t address, uint32_t size, uint32_t count, uint8_t *buffer);
+int arm920t_write_memory(struct target *target,
+	uint32_t address, uint32_t size, uint32_t count, uint8_t *buffer);
+void arm920t_post_debug_entry(struct target *target);
+void arm920t_pre_restore_context(struct target *target);
+	uint32_t arm920t_get_ttb(struct target *target);
+void arm920t_disable_mmu_caches(struct target *target,
+	int mmu, int d_u_cache, int i_cache);
+void arm920t_enable_mmu_caches(struct target *target,
+	int mmu, int d_u_cache, int i_cache);
+
+extern const struct command_registration arm920t_command_handlers[];
 
 #endif /* ARM920T_H */
