@@ -23,39 +23,54 @@
 #ifndef ARM_JTAG
 #define ARM_JTAG
 
-#include "jtag.h"
+#include <jtag/jtag.h>
 
-typedef struct arm_jtag_s
+struct arm_jtag
 {
-	jtag_tap_t *tap;
+	struct jtag_tap *tap;
 
 	uint32_t scann_size;
 	uint32_t scann_instr;
 	uint32_t cur_scan_chain;
 
 	uint32_t intest_instr;
-} arm_jtag_t;
+};
 
-extern int arm_jtag_set_instr(arm_jtag_t *jtag_info, uint32_t new_instr, void *verify_capture);
-extern int arm_jtag_scann(arm_jtag_t *jtag_info, uint32_t new_scan_chain);
-extern int arm_jtag_setup_connection(arm_jtag_t *jtag_info);
+int arm_jtag_set_instr_inner(struct arm_jtag *jtag_info, uint32_t new_instr,  void *no_verify_capture);
+static inline int arm_jtag_set_instr(struct arm_jtag *jtag_info,
+		uint32_t new_instr, void *no_verify_capture)
+{
+	/* inline most common code path */
+	struct jtag_tap *tap;
+	tap = jtag_info->tap;
+	if (tap == NULL)
+		return ERROR_FAIL;
 
-/* JTAG buffers to host, be and le buffers, flipping variants */
-int arm_jtag_buf_to_u32_flip(uint8_t *in_buf, void *priv, struct scan_field_s *field);
-int arm_jtag_buf_to_le32_flip(uint8_t *in_buf, void *priv, struct scan_field_s *field);
-int arm_jtag_buf_to_le16_flip(uint8_t *in_buf, void *priv, struct scan_field_s *field);
-int arm_jtag_buf_to_be32_flip(uint8_t *in_buf, void *priv, struct scan_field_s *field);
-int arm_jtag_buf_to_be16_flip(uint8_t *in_buf, void *priv, struct scan_field_s *field);
-int arm_jtag_buf_to_8_flip(uint8_t *in_buf, void *priv, struct scan_field_s *field);
+	if (buf_get_u32(tap->cur_instr, 0, tap->ir_length) != new_instr)
+	{
+		return arm_jtag_set_instr_inner(jtag_info, new_instr, no_verify_capture);
+	}
 
-/* JTAG buffers to host, be and le buffers */
-int arm_jtag_buf_to_u32(uint8_t *in_buf, void *priv, struct scan_field_s *field);
-int arm_jtag_buf_to_le32(uint8_t *in_buf, void *priv, struct scan_field_s *field);
-int arm_jtag_buf_to_le16(uint8_t *in_buf, void *priv, struct scan_field_s *field);
-int arm_jtag_buf_to_be32(uint8_t *in_buf, void *priv, struct scan_field_s *field);
-int arm_jtag_buf_to_be16(uint8_t *in_buf, void *priv, struct scan_field_s *field);
-int arm_jtag_buf_to_8(uint8_t *in_buf, void *priv, struct scan_field_s *field);
+	return ERROR_OK;
 
+}
+
+
+int arm_jtag_scann_inner(struct arm_jtag *jtag_info, uint32_t new_scan_chain);
+static inline int arm_jtag_scann(struct arm_jtag *jtag_info, uint32_t new_scan_chain)
+{
+	/* inline most common code path */
+	int retval = ERROR_OK;
+	if (jtag_info->cur_scan_chain != new_scan_chain)
+	{
+		return arm_jtag_scann_inner(jtag_info, new_scan_chain);
+	}
+
+	return retval;
+}
+
+
+int arm_jtag_setup_connection(struct arm_jtag *jtag_info);
 
 /* use this as a static so we can inline it in -O3 and refer to it via a pointer  */
 static __inline__ void arm7flip32(jtag_callback_data_t arg)
