@@ -20,9 +20,9 @@
 #ifndef ARM_SIMULATOR_H
 #define ARM_SIMULATOR_H
 
-#include "types.h"
+#include <helper/types.h>
 
-struct target_s;
+struct target;
 
 struct arm_sim_interface
 {
@@ -32,17 +32,16 @@ struct arm_sim_interface
 	uint32_t (*get_reg_mode)(struct arm_sim_interface *sim, int reg);
 	void (*set_reg_mode)(struct arm_sim_interface *sim, int reg, uint32_t value);
 	uint32_t (*get_cpsr)(struct arm_sim_interface *sim, int pos, int bits);
-	enum armv4_5_state (*get_state)(struct arm_sim_interface *sim);
-	void (*set_state)(struct arm_sim_interface *sim, enum armv4_5_state mode);
-	enum armv4_5_mode (*get_mode)(struct arm_sim_interface *sim);
+	enum arm_state (*get_state)(struct arm_sim_interface *sim);
+	void (*set_state)(struct arm_sim_interface *sim, enum arm_state mode);
+	enum arm_mode (*get_mode)(struct arm_sim_interface *sim);
 };
 
-
 /* armv4_5 version */
-extern int arm_simulate_step(struct target_s *target, uint32_t *dry_run_pc);
+int arm_simulate_step(struct target *target, uint32_t *dry_run_pc);
 
 /* a generic arm simulator. Caller must implement the sim interface */
-extern int arm_simulate_step_core(target_t *target, uint32_t *dry_run_pc, struct arm_sim_interface *sim);
-
+int arm_simulate_step_core(struct target *target,
+		uint32_t *dry_run_pc, struct arm_sim_interface *sim);
 
 #endif /* ARM_SIMULATOR_H */
