@@ -22,7 +22,7 @@
 #endif
 
 #include "target.h"
-#include "log.h"
+#include <helper/log.h>
 #include "breakpoints.h"
 
 
@@ -42,10 +42,11 @@ static char *watchpoint_rw_strings[] =
 // monotonic counter/id-number for breakpoints and watch points
 static int bpwp_unique_id;
 
-int breakpoint_add(target_t *target, uint32_t address, uint32_t length, enum breakpoint_type type)
+int breakpoint_add(struct target *target, uint32_t address, uint32_t length, enum breakpoint_type type)
 {
-	breakpoint_t *breakpoint = target->breakpoints;
-	breakpoint_t **breakpoint_p = &target->breakpoints;
+	struct breakpoint *breakpoint = target->breakpoints;
+	struct breakpoint **breakpoint_p = &target->breakpoints;
+	char *reason;
 	int retval;
 	int n;
 
@@ -53,7 +54,11 @@ int breakpoint_add(target_t *target, uint32_t address, uint32_t length, enum bre
 	while (breakpoint)
 	{
 		n++;
-		if (breakpoint->address == address){
+		if (breakpoint->address == address) {
+			/* FIXME don't assume "same address" means "same
+			 * breakpoint" ... check all the parameters before
+			 * succeeding.
+			 */
 			LOG_DEBUG("Duplicate Breakpoint address: 0x%08" PRIx32 " (BP %d)",
 				  address, breakpoint->unique_id );
 			return ERROR_OK;
@@ -62,7 +67,7 @@ int breakpoint_add(target_t *target, uint32_t address, uint32_t length, enum bre
 		breakpoint = breakpoint->next;
 	}
 
-	(*breakpoint_p) = malloc(sizeof(breakpoint_t));
+	(*breakpoint_p) = malloc(sizeof(struct breakpoint));
 	(*breakpoint_p)->address = address;
 	(*breakpoint_p)->length = length;
 	(*breakpoint_p)->type = type;
@@ -71,31 +76,24 @@ int breakpoint_add(target_t *target, uint32_t address, uint32_t length, enum bre
 	(*breakpoint_p)->next = NULL;
 	(*breakpoint_p)->unique_id = bpwp_unique_id++;
 
-	if ((retval = target_add_breakpoint(target, *breakpoint_p)) != ERROR_OK)
-	{
-		switch (retval)
-		{
-			case ERROR_TARGET_RESOURCE_NOT_AVAILABLE:
-				LOG_INFO("can't add %s breakpoint, resource not available (BPID=%d)",
-					 breakpoint_type_strings[(*breakpoint_p)->type],
-					 (*breakpoint_p)->unique_id );
-
-				free((*breakpoint_p)->orig_instr);
-				free(*breakpoint_p);
-				*breakpoint_p = NULL;
-				return retval;
-				break;
-			case ERROR_TARGET_NOT_HALTED:
-				LOG_INFO("can't add breakpoint while target is running (BPID: %d)",
-						 (*breakpoint_p)->unique_id );
-				free((*breakpoint_p)->orig_instr);
-				free(*breakpoint_p);
-				*breakpoint_p = NULL;
-				return retval;
-				break;
-			default:
-				break;
-		}
+	retval = target_add_breakpoint(target, *breakpoint_p);
+	switch (retval) {
+	case ERROR_OK:
+		break;
+	case ERROR_TARGET_RESOURCE_NOT_AVAILABLE:
+		reason = "resource not available";
+		goto fail;
+	case ERROR_TARGET_NOT_HALTED:
+		reason = "target running";
+		goto fail;
+	default:
+		reason = "unknown reason";
+fail:
+		LOG_ERROR("can't add breakpoint: %s", reason);
+		free((*breakpoint_p)->orig_instr);
+		free(*breakpoint_p);
+		*breakpoint_p = NULL;
+		return retval;
 	}
 
 	LOG_DEBUG("added %s breakpoint at 0x%8.8" PRIx32 " of length 0x%8.8x, (BPID: %d)",
@@ -107,10 +105,11 @@ int breakpoint_add(target_t *target, uint32_t address, uint32_t length, enum bre
 }
 
 /* free up a breakpoint */
-static void breakpoint_free(target_t *target, breakpoint_t *breakpoint_remove)
+static void breakpoint_free(struct target *target, struct breakpoint *breakpoint_remove)
 {
-	breakpoint_t *breakpoint = target->breakpoints;
-	breakpoint_t **breakpoint_p = &target->breakpoints;
+	struct breakpoint *breakpoint = target->breakpoints;
+	struct breakpoint **breakpoint_p = &target->breakpoints;
+	int retval;
 
 	while (breakpoint)
 	{
@@ -123,18 +122,18 @@ static void breakpoint_free(target_t *target, breakpoint_t *breakpoint_remove)
 	if (breakpoint == NULL)
 		return;
 
-	target_remove_breakpoint(target, breakpoint);
+	retval = target_remove_breakpoint(target, breakpoint);
 
-	LOG_DEBUG("BPID: %d", breakpoint->unique_id );
+	LOG_DEBUG("free BPID: %d --> %d", breakpoint->unique_id, retval);
 	(*breakpoint_p) = breakpoint->next;
 	free(breakpoint->orig_instr);
 	free(breakpoint);
 }
 
-void breakpoint_remove(target_t *target, uint32_t address)
+void breakpoint_remove(struct target *target, uint32_t address)
 {
-	breakpoint_t *breakpoint = target->breakpoints;
-	breakpoint_t **breakpoint_p = &target->breakpoints;
+	struct breakpoint *breakpoint = target->breakpoints;
+	struct breakpoint **breakpoint_p = &target->breakpoints;
 
 	while (breakpoint)
 	{
@@ -154,19 +153,21 @@ void breakpoint_remove(target_t *target, uint32_t address)
 	}
 }
 
-void breakpoint_clear_target(target_t *target)
+void breakpoint_clear_target(struct target *target)
 {
-	breakpoint_t *breakpoint;
-	LOG_DEBUG("Delete all breakpoints for target: %s", target_get_name( target ));
+	struct breakpoint *breakpoint;
+
+	LOG_DEBUG("Delete all breakpoints for target: %s",
+			target_name(target));
 	while ((breakpoint = target->breakpoints) != NULL)
 	{
 		breakpoint_free(target, breakpoint);
 	}
 }
 
-breakpoint_t* breakpoint_find(target_t *target, uint32_t address)
+struct breakpoint* breakpoint_find(struct target *target, uint32_t address)
 {
-	breakpoint_t *breakpoint = target->breakpoints;
+	struct breakpoint *breakpoint = target->breakpoints;
 
 	while (breakpoint)
 	{
@@ -178,69 +179,78 @@ breakpoint_t* breakpoint_find(target_t *target, uint32_t address)
 	return NULL;
 }
 
-int watchpoint_add(target_t *target, uint32_t address, uint32_t length, enum watchpoint_rw rw, uint32_t value, uint32_t mask)
+int watchpoint_add(struct target *target, uint32_t address, uint32_t length,
+		enum watchpoint_rw rw, uint32_t value, uint32_t mask)
 {
-	watchpoint_t *watchpoint = target->watchpoints;
-	watchpoint_t **watchpoint_p = &target->watchpoints;
+	struct watchpoint *watchpoint = target->watchpoints;
+	struct watchpoint **watchpoint_p = &target->watchpoints;
 	int retval;
+	char *reason;
 
 	while (watchpoint)
 	{
-		if (watchpoint->address == address)
+		if (watchpoint->address == address) {
+			if (watchpoint->length != length
+					|| watchpoint->value != value
+					|| watchpoint->mask != mask
+					|| watchpoint->rw != rw) {
+				LOG_ERROR("address 0x%8.8" PRIx32
+						"already has watchpoint %d",
+						address, watchpoint->unique_id);
+				return ERROR_FAIL;
+			}
+
+			/* ignore duplicate watchpoint */
 			return ERROR_OK;
+		}
 		watchpoint_p = &watchpoint->next;
 		watchpoint = watchpoint->next;
 	}
 
-	(*watchpoint_p) = malloc(sizeof(watchpoint_t));
+	(*watchpoint_p) = calloc(1, sizeof(struct watchpoint));
 	(*watchpoint_p)->address = address;
 	(*watchpoint_p)->length = length;
 	(*watchpoint_p)->value = value;
 	(*watchpoint_p)->mask = mask;
 	(*watchpoint_p)->rw = rw;
-	(*watchpoint_p)->set = 0;
-	(*watchpoint_p)->next = NULL;
 	(*watchpoint_p)->unique_id = bpwp_unique_id++;
 
-	if ((retval = target_add_watchpoint(target, *watchpoint_p)) != ERROR_OK)
-	{
-		switch (retval)
-		{
-			case ERROR_TARGET_RESOURCE_NOT_AVAILABLE:
-				LOG_INFO("can't add %s watchpoint, resource not available (WPID: %d)",
-					 watchpoint_rw_strings[(*watchpoint_p)->rw],
-					 (*watchpoint_p)->unique_id );
-				free (*watchpoint_p);
-				*watchpoint_p = NULL;
-				return retval;
-				break;
-			case ERROR_TARGET_NOT_HALTED:
-				LOG_INFO("can't add watchpoint while target is running (WPID: %d)",
-						 (*watchpoint_p)->unique_id );
-				free (*watchpoint_p);
-				*watchpoint_p = NULL;
-				return retval;
-				break;
-			default:
-				LOG_ERROR("unknown error");
-				exit(-1);
-				break;
-		}
+	retval = target_add_watchpoint(target, *watchpoint_p);
+	switch (retval) {
+	case ERROR_OK:
+		break;
+	case ERROR_TARGET_RESOURCE_NOT_AVAILABLE:
+		reason = "resource not available";
+		goto bye;
+	case ERROR_TARGET_NOT_HALTED:
+		reason = "target running";
+		goto bye;
+	default:
+		reason = "unrecognized error";
+bye:
+		LOG_ERROR("can't add %s watchpoint at 0x%8.8" PRIx32 ", %s",
+			 watchpoint_rw_strings[(*watchpoint_p)->rw],
+			 address, reason);
+		free (*watchpoint_p);
+		*watchpoint_p = NULL;
+		return retval;
 	}
 
-	LOG_DEBUG("added %s watchpoint at 0x%8.8" PRIx32 " of length 0x%8.8x (WPID: %d)",
-			  watchpoint_rw_strings[(*watchpoint_p)->rw],
-			  (*watchpoint_p)->address,
-			  (*watchpoint_p)->length,
-			  (*watchpoint_p)->unique_id );
+	LOG_DEBUG("added %s watchpoint at 0x%8.8" PRIx32
+			" of length 0x%8.8" PRIx32 " (WPID: %d)",
+			watchpoint_rw_strings[(*watchpoint_p)->rw],
+			(*watchpoint_p)->address,
+			(*watchpoint_p)->length,
+			(*watchpoint_p)->unique_id );
 
 	return ERROR_OK;
 }
 
-static void watchpoint_free(target_t *target, watchpoint_t *watchpoint_remove)
+static void watchpoint_free(struct target *target, struct watchpoint *watchpoint_remove)
 {
-	watchpoint_t *watchpoint = target->watchpoints;
-	watchpoint_t **watchpoint_p = &target->watchpoints;
+	struct watchpoint *watchpoint = target->watchpoints;
+	struct watchpoint **watchpoint_p = &target->watchpoints;
+	int retval;
 
 	while (watchpoint)
 	{
@@ -252,16 +262,16 @@ static void watchpoint_free(target_t *target, watchpoint_t *watchpoint_remove)
 
 	if (watchpoint == NULL)
 		return;
-	target_remove_watchpoint(target, watchpoint);
-	LOG_DEBUG("WPID: %d", watchpoint->unique_id );
+	retval = target_remove_watchpoint(target, watchpoint);
+	LOG_DEBUG("free WPID: %d --> %d", watchpoint->unique_id, retval);
 	(*watchpoint_p) = watchpoint->next;
 	free(watchpoint);
 }
 
-void watchpoint_remove(target_t *target, uint32_t address)
+void watchpoint_remove(struct target *target, uint32_t address)
 {
-	watchpoint_t *watchpoint = target->watchpoints;
-	watchpoint_t **watchpoint_p = &target->watchpoints;
+	struct watchpoint *watchpoint = target->watchpoints;
+	struct watchpoint **watchpoint_p = &target->watchpoints;
 
 	while (watchpoint)
 	{
@@ -281,10 +291,12 @@ void watchpoint_remove(target_t *target, uint32_t address)
 	}
 }
 
-void watchpoint_clear_target(target_t *target)
+void watchpoint_clear_target(struct target *target)
 {
-	watchpoint_t *watchpoint;
-	LOG_DEBUG("Delete all watchpoints for target: %s", target_get_name( target ));
+	struct watchpoint *watchpoint;
+
+	LOG_DEBUG("Delete all watchpoints for target: %s",
+			target_name(target));
 	while ((watchpoint = target->watchpoints) != NULL)
 	{
 		watchpoint_free(target, watchpoint);
