@@ -26,7 +26,7 @@
 #ifndef IMAGE_H
 #define IMAGE_H
 
-#include "fileio.h"
+#include <helper/fileio.h>
 
 #ifdef HAVE_ELF_H
 #include <elf.h>
@@ -37,7 +37,7 @@
 
 #define IMAGE_MEMORY_CACHE_SIZE		(2048)
 
-typedef enum image_type
+enum image_type
 {
 	IMAGE_BINARY,	/* plain binary */
 	IMAGE_IHEX,		/* intel hex-record format */
@@ -45,67 +45,71 @@ typedef enum image_type
 	IMAGE_ELF,		/* ELF binary */
 	IMAGE_SRECORD,	/* motorola s19 */
 	IMAGE_BUILDER,	/* when building a new image */
-} image_type_t;
+};
 
-typedef struct image_section_s
+struct imageection
 {
 	uint32_t base_address;
 	uint32_t size;
 	int flags;
 	void *private;		/* private data */
-} image_section_t;
+};
 
-typedef struct image_s
+struct image
 {
-	image_type_t type;		/* image type (plain, ihex, ...) */
+	enum image_type type;		/* image type (plain, ihex, ...) */
 	void *type_private;		/* type private data */
 	int num_sections;		/* number of sections contained in the image */
-	image_section_t *sections;	/* array of sections */
+	struct imageection *sections;	/* array of sections */
 	int base_address_set;	/* whether the image has a base address set (for relocation purposes) */
 	int base_address;		/* base address, if one is set */
 	int start_address_set;	/* whether the image has a start address (entry point) associated */
 	uint32_t start_address;		/* start address, if one is set */
-} image_t;
+};
 
-typedef struct image_binary_s
+struct image_binary
 {
-	fileio_t fileio;
-} image_binary_t;
+	struct fileio fileio;
+};
 
-typedef struct image_ihex_s
+struct image_ihex
 {
-	fileio_t fileio;
+	struct fileio fileio;
 	uint8_t *buffer;
-} image_ihex_t;
+};
 
-typedef struct image_memory_s
+struct image_memory
 {
-	struct target_s *target;
+	struct target *target;
 	uint8_t *cache;
 	uint32_t cache_address;
-} image_memory_t;
+};
 
-typedef struct fileio_elf_s
+struct image_elf
 {
-	fileio_t fileio;
+	struct fileio fileio;
 	Elf32_Ehdr *header;
 	Elf32_Phdr *segments;
 	uint32_t segment_count;
 	uint8_t endianness;
-} image_elf_t;
+};
 
-typedef struct image_mot_s
+struct image_mot
 {
-	fileio_t fileio;
+	struct fileio fileio;
 	uint8_t *buffer;
-} image_mot_t;
+};
 
-extern int image_open(image_t *image, char *url, char *type_string);
-extern int image_read_section(image_t *image, int section, uint32_t offset, uint32_t size, uint8_t *buffer, uint32_t *size_read);
-extern void image_close(image_t *image);
-extern int image_add_section(image_t *image, uint32_t base, uint32_t size, int flags, uint8_t *data);
+int image_open(struct image *image, const char *url, const char *type_string);
+int image_read_section(struct image *image, int section, uint32_t offset,
+		uint32_t size, uint8_t *buffer, size_t *size_read);
+void image_close(struct image *image);
 
-extern int image_calculate_checksum(uint8_t* buffer, uint32_t nbytes, uint32_t* checksum);
+int image_add_section(struct image *image, uint32_t base, uint32_t size,
+		int flags, uint8_t *data);
+
+int image_calculate_checksum(uint8_t* buffer, uint32_t nbytes,
+		uint32_t* checksum);
 
 #define ERROR_IMAGE_FORMAT_ERROR	(-1400)
 #define ERROR_IMAGE_TYPE_UNKNOWN	(-1401)
