@@ -4,6 +4,8 @@
  *                                                                         *
  *   Copyright (C) 2008 by David T.L. Wong                                 *
  *                                                                         *
+ *   Copyright (C) 2009 by David N. Claffey <dnclaffey@gmail.com>          *
+ *                                                                         *
  *   This program is free software; you can redistribute it and/or modify  *
  *   it under the terms of the GNU General Public License as published by  *
  *   the Free Software Foundation; either version 2 of the License, or     *
@@ -26,10 +28,9 @@
 #include "mips32.h"
 #include "mips_ejtag.h"
 
-
-int mips_ejtag_set_instr(mips_ejtag_t *ejtag_info, int new_instr, void *delete_me_and_submit_patch)
+int mips_ejtag_set_instr(struct mips_ejtag *ejtag_info, int new_instr, void *delete_me_and_submit_patch)
 {
-	jtag_tap_t *tap;
+	struct jtag_tap *tap;
 
 	tap = ejtag_info->tap;
 	if (tap == NULL)
@@ -37,7 +38,7 @@ int mips_ejtag_set_instr(mips_ejtag_t *ejtag_info, int new_instr, void *delete_m
 
 	if (buf_get_u32(tap->cur_instr, 0, tap->ir_length) != (uint32_t)new_instr)
 	{
-		scan_field_t field;
+		struct scan_field field;
 		uint8_t t[4];
 
 		field.tap = tap;
@@ -52,9 +53,9 @@ int mips_ejtag_set_instr(mips_ejtag_t *ejtag_info, int new_instr, void *delete_m
 	return ERROR_OK;
 }
 
-int mips_ejtag_get_idcode(mips_ejtag_t *ejtag_info, uint32_t *idcode)
+int mips_ejtag_get_idcode(struct mips_ejtag *ejtag_info, uint32_t *idcode)
 {
-	scan_field_t field;
+	struct scan_field field;
 
 	jtag_set_end_state(TAP_IDLE);
 
@@ -75,9 +76,9 @@ int mips_ejtag_get_idcode(mips_ejtag_t *ejtag_info, uint32_t *idcode)
 	return ERROR_OK;
 }
 
-int mips_ejtag_get_impcode(mips_ejtag_t *ejtag_info, uint32_t *impcode)
+int mips_ejtag_get_impcode(struct mips_ejtag *ejtag_info, uint32_t *impcode)
 {
-	scan_field_t field;
+	struct scan_field field;
 
 	jtag_set_end_state(TAP_IDLE);
 
@@ -98,14 +99,14 @@ int mips_ejtag_get_impcode(mips_ejtag_t *ejtag_info, uint32_t *impcode)
 	return ERROR_OK;
 }
 
-int mips_ejtag_drscan_32(mips_ejtag_t *ejtag_info, uint32_t *data)
+int mips_ejtag_drscan_32(struct mips_ejtag *ejtag_info, uint32_t *data)
 {
-	jtag_tap_t *tap;
+	struct jtag_tap *tap;
 	tap  = ejtag_info->tap;
 
 	if (tap == NULL)
 		return ERROR_FAIL;
-	scan_field_t field;
+	struct scan_field field;
 	uint8_t t[4], r[4];
 	int retval;
 
@@ -130,59 +131,55 @@ int mips_ejtag_drscan_32(mips_ejtag_t *ejtag_info, uint32_t *data)
 	return ERROR_OK;
 }
 
-int mips_ejtag_step_enable(mips_ejtag_t *ejtag_info)
+int mips_ejtag_step_enable(struct mips_ejtag *ejtag_info)
 {
 	uint32_t code[] = {
 			MIPS32_MTC0(1,31,0),			/* move $1 to COP0 DeSave */
 			MIPS32_MFC0(1,23,0),			/* move COP0 Debug to $1 */
 			MIPS32_ORI(1,1,0x0100),			/* set SSt bit in debug reg */
 			MIPS32_MTC0(1,23,0),			/* move $1 to COP0 Debug */
+			MIPS32_B(NEG16(5)),
 			MIPS32_MFC0(1,31,0),			/* move COP0 DeSave to $1 */
-			MIPS32_NOP,
-			MIPS32_B(NEG16(7)),
-			MIPS32_NOP,
 	};
 
-	mips32_pracc_exec(ejtag_info, sizeof(code)/sizeof(code[0]), code, \
+	mips32_pracc_exec(ejtag_info, ARRAY_SIZE(code), code, \
 		0, NULL, 0, NULL, 1);
 
 	return ERROR_OK;
 }
-int mips_ejtag_step_disable(mips_ejtag_t *ejtag_info)
+int mips_ejtag_step_disable(struct mips_ejtag *ejtag_info)
 {
 	uint32_t code[] = {
 			MIPS32_MTC0(15,31,0),							/* move $15 to COP0 DeSave */
-			MIPS32_LUI(15,UPPER16(MIPS32_PRACC_STACK)), 	/* $15 = MIPS32_PRACC_STACK */
+			MIPS32_LUI(15,UPPER16(MIPS32_PRACC_STACK)),		/* $15 = MIPS32_PRACC_STACK */
 			MIPS32_ORI(15,15,LOWER16(MIPS32_PRACC_STACK)),
-			MIPS32_SW(1,0,15), 								/* sw $1,($15) */
-			MIPS32_SW(2,0,15), 								/* sw $2,($15) */
+			MIPS32_SW(1,0,15),								/* sw $1,($15) */
+			MIPS32_SW(2,0,15),								/* sw $2,($15) */
 			MIPS32_MFC0(1,23,0),							/* move COP0 Debug to $1 */
-			MIPS32_LUI(2,0xFFFF), 							/* $2 = 0xfffffeff */
+			MIPS32_LUI(2,0xFFFF),							/* $2 = 0xfffffeff */
 			MIPS32_ORI(2,2,0xFEFF),
 			MIPS32_AND(1,1,2),
 			MIPS32_MTC0(1,23,0),							/* move $1 to COP0 Debug */
 			MIPS32_LW(2,0,15),
 			MIPS32_LW(1,0,15),
+			MIPS32_B(NEG16(13)),
 			MIPS32_MFC0(15,31,0),							/* move COP0 DeSave to $15 */
-			MIPS32_NOP,
-			MIPS32_B(NEG16(15)),
-			MIPS32_NOP,
 	};
 
-	mips32_pracc_exec(ejtag_info, sizeof(code)/sizeof(code[0]), code, \
+	mips32_pracc_exec(ejtag_info, ARRAY_SIZE(code), code, \
 		0, NULL, 0, NULL, 1);
 
 	return ERROR_OK;
 }
 
-int mips_ejtag_config_step(mips_ejtag_t *ejtag_info, int enable_step)
+int mips_ejtag_config_step(struct mips_ejtag *ejtag_info, int enable_step)
 {
 	if (enable_step)
 		return mips_ejtag_step_enable(ejtag_info);
 	return mips_ejtag_step_disable(ejtag_info);
 }
 
-int mips_ejtag_enter_debug(mips_ejtag_t *ejtag_info)
+int mips_ejtag_enter_debug(struct mips_ejtag *ejtag_info)
 {
 	uint32_t ejtag_ctrl;
 	jtag_set_end_state(TAP_IDLE);
@@ -202,7 +199,7 @@ int mips_ejtag_enter_debug(mips_ejtag_t *ejtag_info)
 	return ERROR_OK;
 }
 
-int mips_ejtag_exit_debug(mips_ejtag_t *ejtag_info)
+int mips_ejtag_exit_debug(struct mips_ejtag *ejtag_info)
 {
 	uint32_t inst;
 	inst = MIPS32_DRET;
@@ -213,34 +210,32 @@ int mips_ejtag_exit_debug(mips_ejtag_t *ejtag_info)
 	return ERROR_OK;
 }
 
-int mips_ejtag_read_debug(mips_ejtag_t *ejtag_info, uint32_t* debug_reg)
+int mips_ejtag_read_debug(struct mips_ejtag *ejtag_info, uint32_t* debug_reg)
 {
 	/* read ejtag ECR */
 	uint32_t code[] = {
 			MIPS32_MTC0(15,31,0),							/* move $15 to COP0 DeSave */
-			MIPS32_LUI(15,UPPER16(MIPS32_PRACC_STACK)), 	/* $15 = MIPS32_PRACC_STACK */
+			MIPS32_LUI(15,UPPER16(MIPS32_PRACC_STACK)),		/* $15 = MIPS32_PRACC_STACK */
 			MIPS32_ORI(15,15,LOWER16(MIPS32_PRACC_STACK)),
-			MIPS32_SW(1,0,15), 								/* sw $1,($15) */
-			MIPS32_SW(2,0,15), 								/* sw $2,($15) */
-			MIPS32_LUI(1,UPPER16(MIPS32_PRACC_PARAM_OUT)), 	/* $1 = MIPS32_PRACC_PARAM_OUT */
+			MIPS32_SW(1,0,15),								/* sw $1,($15) */
+			MIPS32_SW(2,0,15),								/* sw $2,($15) */
+			MIPS32_LUI(1,UPPER16(MIPS32_PRACC_PARAM_OUT)),	/* $1 = MIPS32_PRACC_PARAM_OUT */
 			MIPS32_ORI(1,1,LOWER16(MIPS32_PRACC_PARAM_OUT)),
 			MIPS32_MFC0(2,23,0),							/* move COP0 Debug to $2 */
 			MIPS32_SW(2,0,1),
 			MIPS32_LW(2,0,15),
 			MIPS32_LW(1,0,15),
+			MIPS32_B(NEG16(12)),
 			MIPS32_MFC0(15,31,0),							/* move COP0 DeSave to $15 */
-			MIPS32_NOP,
-			MIPS32_B(NEG16(14)),
-			MIPS32_NOP,
 	};
 
-	mips32_pracc_exec(ejtag_info, sizeof(code)/sizeof(code[0]), code, \
+	mips32_pracc_exec(ejtag_info, ARRAY_SIZE(code), code, \
 		0, NULL, 1, debug_reg, 1);
 
 	return ERROR_OK;
 }
 
-int mips_ejtag_init(mips_ejtag_t *ejtag_info)
+int mips_ejtag_init(struct mips_ejtag *ejtag_info)
 {
 	uint32_t ejtag_version;
 
@@ -269,16 +264,15 @@ int mips_ejtag_init(mips_ejtag_t *ejtag_info)
 			break;
 	}
 	LOG_DEBUG("EJTAG: features:%s%s%s%s%s%s%s",
-		ejtag_info->impcode & (1 << 28) ? " R3k":    " R4k",
-		ejtag_info->impcode & (1 << 24) ? " DINT":   "",
-		ejtag_info->impcode & (1 << 22) ? " ASID_8": "",
-		ejtag_info->impcode & (1 << 21) ? " ASID_6": "",
-		ejtag_info->impcode & (1 << 16) ? " MIPS16": "",
-		ejtag_info->impcode & (1 << 14) ? " noDMA":  " DMA",
-		ejtag_info->impcode & (1 << 0)  ? " MIPS64": " MIPS32"
-);
+		ejtag_info->impcode & EJTAG_IMP_R3K ? " R3k" : " R4k",
+		ejtag_info->impcode & EJTAG_IMP_DINT ? " DINT" : "",
+		ejtag_info->impcode & (1 << 22) ? " ASID_8" : "",
+		ejtag_info->impcode & (1 << 21) ? " ASID_6" : "",
+		ejtag_info->impcode & EJTAG_IMP_MIPS16 ? " MIPS16" : "",
+		ejtag_info->impcode & EJTAG_IMP_NODMA ? " noDMA" : " DMA",
+		ejtag_info->impcode & EJTAG_DCR_MIPS64  ? " MIPS64" : " MIPS32");
 
-	if ((ejtag_info->impcode & (1 << 14)) == 0)
+	if ((ejtag_info->impcode & EJTAG_IMP_NODMA) == 0)
 		LOG_DEBUG("EJTAG: DMA Access Mode Support Enabled");
 
 	/* set initial state for ejtag control reg */
@@ -286,3 +280,42 @@ int mips_ejtag_init(mips_ejtag_t *ejtag_info)
 
 	return ERROR_OK;
 }
+
+int mips_ejtag_fastdata_scan(struct mips_ejtag *ejtag_info, int write, uint32_t *data)
+{
+	struct jtag_tap *tap;
+	tap = ejtag_info->tap;
+
+	if (tap == NULL)
+		return ERROR_FAIL;
+
+	struct scan_field fields[2];
+	uint8_t spracc = 0;
+	uint8_t t[4] = {0, 0, 0, 0};
+
+	/* fastdata 1-bit register */
+	fields[0].tap = tap;
+	fields[0].num_bits = 1;
+	fields[0].out_value = &spracc;
+	fields[0].in_value = NULL;
+
+	/* processor access data register 32 bit */
+	fields[1].tap = tap;
+	fields[1].num_bits = 32;
+	fields[1].out_value = t;
+
+	if (write)
+	{
+		fields[1].in_value = NULL;
+		buf_set_u32(t, 0, 32, *data);
+	}
+	else
+	{
+		fields[1].in_value = (uint8_t *) data;
+	}
+
+	jtag_add_dr_scan(2, fields, jtag_get_end_state());
+	keep_alive();
+
+	return ERROR_OK;
+}
