@@ -24,16 +24,25 @@
 #include "config.h"
 #endif
 
+#include <helper/types.h>
 #include "register.h"
-#include "log.h"
+#include <helper/log.h>
 
+/**
+ * @file
+ * Holds utilities to work with register caches.
+ *
+ * OpenOCD uses machine registers internally, and exposes them by name
+ * to Tcl scripts.  Sets of related registers are grouped into caches.
+ * For example, a CPU core will expose a set of registers, and there
+ * may be separate registers associated with debug or trace modules.
+ */
 
-reg_arch_type_t *reg_arch_types = NULL;
-
-reg_t* register_get_by_name(reg_cache_t *first, char *name, int search_all)
+struct reg* register_get_by_name(struct reg_cache *first,
+		const char *name, bool search_all)
 {
-	int i;
-	reg_cache_t *cache = first;
+	unsigned i;
+	struct reg_cache *cache = first;
 
 	while (cache)
 	{
@@ -52,9 +61,9 @@ reg_t* register_get_by_name(reg_cache_t *first, char *name, int search_all)
 	return NULL;
 }
 
-reg_cache_t** register_get_last_cache_p(reg_cache_t **first)
+struct reg_cache** register_get_last_cache_p(struct reg_cache **first)
 {
-	reg_cache_t **cache_p = first;
+	struct reg_cache **cache_p = first;
 
 	if (*cache_p)
 		while (*cache_p)
@@ -65,50 +74,23 @@ reg_cache_t** register_get_last_cache_p(reg_cache_t **first)
 	return cache_p;
 }
 
-int register_reg_arch_type(int (*get)(reg_t *reg), int (*set)(reg_t *reg, uint8_t *buf))
+/** Marks the contents of the register cache as invalid (and clean). */
+void register_cache_invalidate(struct reg_cache *cache)
 {
-	reg_arch_type_t** arch_type_p = &reg_arch_types;
-	int id = 0;
+	struct reg *reg = cache->reg_list;
 
-	if (*arch_type_p)
-	{
-		while (*arch_type_p)
-		{
-			id = (*arch_type_p)->id;
-			arch_type_p = &((*arch_type_p)->next);
-		}
+	for (unsigned n = cache->num_regs; n != 0; n--, reg++) {
+		reg->valid = 0;
+		reg->dirty = 0;
 	}
-
-	(*arch_type_p) = malloc(sizeof(reg_arch_type_t));
-	(*arch_type_p)->id = id + 1;
-	(*arch_type_p)->set = set;
-	(*arch_type_p)->get = get;
-	(*arch_type_p)->next = NULL;
-
-	return id + 1;
 }
 
-reg_arch_type_t* register_get_arch_type(int id)
-{
-	reg_arch_type_t *arch_type = reg_arch_types;
-
-	while (arch_type)
-	{
-		if (arch_type->id == id)
-			return arch_type;
-		arch_type = arch_type->next;
-	}
-	LOG_ERROR("BUG: encountered unregistered arch type 0x%08x", id);
-	exit(-1);
-	return NULL;
-}
-
-static int register_get_dummy_core_reg(reg_t *reg)
+static int register_get_dummy_core_reg(struct reg *reg)
 {
 	return ERROR_OK;
 }
 
-static int register_set_dummy_core_reg(reg_t *reg, uint8_t *buf)
+static int register_set_dummy_core_reg(struct reg *reg, uint8_t *buf)
 {
 	reg->dirty = 1;
 	reg->valid = 1;
@@ -116,11 +98,12 @@ static int register_set_dummy_core_reg(reg_t *reg, uint8_t *buf)
 	return ERROR_OK;
 }
 
-void register_init_dummy(reg_t *reg)
-{
-	static int dummy_arch_type = -1;
-	if (dummy_arch_type == -1)
-		dummy_arch_type = register_reg_arch_type(register_get_dummy_core_reg, register_set_dummy_core_reg);
+static const struct reg_arch_type dummy_type = {
+	.get = register_get_dummy_core_reg,
+	.set = register_set_dummy_core_reg,
+};
 
-	reg->arch_type = dummy_arch_type;
+void register_init_dummy(struct reg *reg)
+{
+	reg->type = &dummy_type;
 }
