@@ -23,49 +23,38 @@
 #ifndef REGISTER_H
 #define REGISTER_H
 
-#include "types.h"
+struct target;
 
-struct target_s;
-
-typedef struct bitfield_desc_s
-{
-	char *name;
-	int num_bits;
-} bitfield_desc_t;
-
-typedef struct reg_s
+struct reg
 {
 	char *name;
 	void *value;
-	int dirty;
-	int valid;
+	bool dirty;
+	bool valid;
 	uint32_t size;
-	bitfield_desc_t *bitfield_desc;
-	int num_bitfields;
 	void *arch_info;
-	int arch_type;
-} reg_t;
+	const struct reg_arch_type *type;
+};
 
-typedef struct reg_cache_s
+struct reg_cache
 {
 	char *name;
-	struct reg_cache_s *next;
-	reg_t *reg_list;
-	int num_regs;
-} reg_cache_t;
+	struct reg_cache *next;
+	struct reg *reg_list;
+	unsigned num_regs;
+};
 
-typedef struct reg_arch_type_s
+struct reg_arch_type
 {
-	int id;
-	int (*get)(reg_t *reg);
-	int (*set)(reg_t *reg, uint8_t *buf);
-	struct reg_arch_type_s *next;
-} reg_arch_type_t;
+	int (*get)(struct reg *reg);
+	int (*set)(struct reg *reg, uint8_t *buf);
+};
 
-extern reg_t* register_get_by_name(reg_cache_t *first, char *name, int search_all);
-extern reg_cache_t** register_get_last_cache_p(reg_cache_t **first);
-extern int register_reg_arch_type(int (*get)(reg_t *reg), int (*set)(reg_t *reg, uint8_t *buf));
-extern reg_arch_type_t* register_get_arch_type(int id);
-extern void register_init_dummy(reg_t *reg);
+struct reg* register_get_by_name(struct reg_cache *first,
+		const char *name, bool search_all);
+struct reg_cache** register_get_last_cache_p(struct reg_cache **first);
+void register_cache_invalidate(struct reg_cache *cache);
+
+void register_init_dummy(struct reg *reg);
 
 #endif /* REGISTER_H */
