@@ -2,7 +2,7 @@
  *   Copyright (C) 2005 by Dominic Rath                                    *
  *   Dominic.Rath@gmx.de                                                   *
  *                                                                         *
- *   Copyright (C) 2007,2008,2009 Øyvind Harboe                            *
+ *   Copyright (C) 2007-2010 Øyvind Harboe                                 *
  *   oyvind.harboe@zylin.com                                               *
  *                                                                         *
  *   Copyright (C) 2008 by Spencer Oliver                                  *
@@ -26,13 +26,16 @@
 #ifndef TARGET_H
 #define TARGET_H
 
-#include "breakpoints.h"
-#include "algorithm.h"
-#include "command.h"
+#include <helper/types.h>
+
+struct reg;
+struct trace;
+struct command_context;
+struct breakpoint;
+struct watchpoint;
+struct mem_param;
+struct reg_param;
 
-struct reg_s;
-struct trace_s;
-struct command_context_s;
 
 /*
  * TARGET_UNKNOWN = 0: we don't know anything about the target yet
@@ -57,15 +60,11 @@ enum target_state
 	TARGET_DEBUG_RUNNING = 4,
 };
 
-extern const Jim_Nvp nvp_target_state[];
-
 enum nvp_assert {
 	NVP_DEASSERT,
 	NVP_ASSERT,
 };
 
-extern const Jim_Nvp nvp_assert[];
-
 enum target_reset_mode
 {
 	RESET_UNKNOWN = 0,
@@ -74,8 +73,6 @@ enum target_reset_mode
 	RESET_INIT = 3,		/* reset and halt target out of reset, then run init script */
 };
 
-extern const Jim_Nvp nvp_reset_mode[];
-
 enum target_debug_reason
 {
 	DBG_REASON_DBGRQ = 0,
@@ -87,43 +84,40 @@ enum target_debug_reason
 	DBG_REASON_UNDEFINED = 6
 };
 
-extern const Jim_Nvp nvp_target_debug_reason[];
-
 enum target_endianess
 {
 	TARGET_ENDIAN_UNKNOWN = 0,
 	TARGET_BIG_ENDIAN = 1, TARGET_LITTLE_ENDIAN = 2
 };
 
-extern const Jim_Nvp nvp_target_endian[];
-
-struct target_s;
-
-typedef struct working_area_s
+struct working_area
 {
 	uint32_t address;
 	uint32_t size;
 	int free;
 	uint8_t *backup;
-	struct working_area_s **user;
-	struct working_area_s *next;
-} working_area_t;
+	struct working_area **user;
+	struct working_area *next;
+};
 
-// target_type.h contains the full definitionof struct target_type_s
-struct target_type_s;
-typedef struct target_type_s target_type_t;
-
-/* forward decloration */
-typedef struct target_event_action_s target_event_action_t;
-
-typedef struct target_s
+// target_type.h contains the full definitionof struct targe_type
+struct target
 {
-	target_type_t *type;				/* target type definition (name, access functions) */
+	struct target_type *type;				/* target type definition (name, access functions) */
 	const char *cmd_name;				/* tcl Name of target */
 	int target_number;					/* DO NOT USE!  field to be removed in 2010 */
-	jtag_tap_t *tap;					/* where on the jtag chain is this */
-	const char *variant;				/* what varient of this chip is it? */
-	target_event_action_t *event_action;
+	struct jtag_tap *tap;					/* where on the jtag chain is this */
+	const char *variant;				/* what variant of this chip is it? */
+
+	/**
+	 * Indicates whether this target has been examined.
+	 *
+	 * Do @b not access this field directly, use target_was_examined()
+	 * or target_set_examined().
+	 */
+	bool examined;
+
+	struct target_event_action *event_action;
 
 	int reset_halt;						/* attempt resetting the CPU into the halted mode? */
 	uint32_t working_area;					/* working area (initialized RAM). Evaluated
@@ -134,25 +128,33 @@ typedef struct target_s
 	uint32_t working_area_phys;			/* physical address */
 	uint32_t working_area_size;			/* size in bytes */
 	uint32_t backup_working_area;			/* whether the content of the working area has to be preserved */
-	struct working_area_s *working_areas;/* list of allocated working areas */
+	struct working_area *working_areas;/* list of allocated working areas */
 	enum target_debug_reason debug_reason;/* reason why the target entered debug state */
 	enum target_endianess endianness;	/* target endianess */
 	// also see: target_state_name()
 	enum target_state state;			/* the current backend-state (running, halted, ...) */
-	struct reg_cache_s *reg_cache;		/* the first register cache of the target (core regs) */
-	struct breakpoint_s *breakpoints;	/* list of breakpoints */
-	struct watchpoint_s *watchpoints;	/* list of watchpoints */
-	struct trace_s *trace_info;			/* generic trace information */
-	struct debug_msg_receiver_s *dbgmsg;/* list of debug message receivers */
+	struct reg_cache *reg_cache;		/* the first register cache of the target (core regs) */
+	struct breakpoint *breakpoints;	/* list of breakpoints */
+	struct watchpoint *watchpoints;	/* list of watchpoints */
+	struct trace *trace_info;			/* generic trace information */
+	struct debug_msg_receiver *dbgmsg;/* list of debug message receivers */
 	uint32_t dbg_msg_enabled;				/* debug message status */
 	void *arch_info;					/* architecture specific information */
-	struct target_s *next;				/* next target in list */
+	struct target *next;				/* next target in list */
 
 	int display;						/* display async info in telnet session. Do not display
 										 * lots of halted/resumed info when stepping in debugger. */
 	bool halt_issued;					/* did we transition to halted state? */
 	long long halt_issued_time;			/* Note time when halt was issued */
-} target_t;
+};
+
+/** Returns the instance-specific name of the specified target. */
+static inline const char *target_name(struct target *target)
+{
+	return target->cmd_name;
+}
+
+const char *debug_reason_name(struct target *t);
 
 enum target_event
 {
@@ -161,8 +163,6 @@ enum target_event
 	 * - June/July/Aug 2008
 	 * - Duane Ellis */
 	TARGET_EVENT_OLD_gdb_program_config,
-	TARGET_EVENT_OLD_pre_reset,
-	TARGET_EVENT_OLD_post_reset,
 	TARGET_EVENT_OLD_pre_resume,
 
 	/* allow GDB to do stuff before others handle the halted event,
@@ -185,6 +185,7 @@ enum target_event
 
 	TARGET_EVENT_RESET_START,
 	TARGET_EVENT_RESET_ASSERT_PRE,
+	TARGET_EVENT_RESET_ASSERT,	/* C code uses this instead of SRST */
 	TARGET_EVENT_RESET_ASSERT_POST,
 	TARGET_EVENT_RESET_DEASSERT_PRE,
 	TARGET_EVENT_RESET_DEASSERT_POST,
@@ -210,131 +211,153 @@ enum target_event
 	TARGET_EVENT_GDB_FLASH_WRITE_END,
 };
 
-struct target_event_action_s {
+struct target_event_action {
 	enum target_event event;
-	Jim_Obj *body;
+	struct Jim_Interp *interp;
+	struct Jim_Obj *body;
 	int has_percent;
-	target_event_action_t *next;
- };
+	struct target_event_action *next;
+};
 
-typedef struct target_event_callback_s
+bool target_has_event_action(struct target *target, enum target_event event);
+
+struct target_event_callback
 {
-	int (*callback)(struct target_s *target, enum target_event event, void *priv);
+	int (*callback)(struct target *target, enum target_event event, void *priv);
 	void *priv;
-	struct target_event_callback_s *next;
-} target_event_callback_t;
+	struct target_event_callback *next;
+};
 
-typedef struct target_timer_callback_s
+struct target_timer_callback
 {
 	int (*callback)(void *priv);
 	int time_ms;
 	int periodic;
 	struct timeval when;
 	void *priv;
-	struct target_timer_callback_s *next;
-} target_timer_callback_t;
+	struct target_timer_callback *next;
+};
 
-extern int target_register_commands(struct command_context_s *cmd_ctx);
-extern int target_register_user_commands(struct command_context_s *cmd_ctx);
-extern int target_init(struct command_context_s *cmd_ctx);
-extern int target_examine(void);
-extern int handle_target(void *priv);
-extern int target_process_reset(struct command_context_s *cmd_ctx, enum target_reset_mode reset_mode);
+int target_register_commands(struct command_context *cmd_ctx);
+int target_register_user_commands(struct command_context *cmd_ctx);
+int target_init(struct command_context *cmd_ctx);
+int target_examine(void);
+int target_process_reset(struct command_context *cmd_ctx,
+		enum target_reset_mode reset_mode);
 
-extern int target_register_event_callback(int (*callback)(struct target_s *target, enum target_event event, void *priv), void *priv);
-extern int target_unregister_event_callback(int (*callback)(struct target_s *target, enum target_event event, void *priv), void *priv);
-extern int target_poll(target_t *target);
-extern int target_resume(target_t *target, int current, uint32_t address, int handle_breakpoints, int debug_execution);
-extern int target_halt(target_t *target);
-extern int target_call_event_callbacks(target_t *target, enum target_event event);
+int target_register_event_callback(
+		int (*callback)(struct target *target,
+				enum target_event event, void *priv),
+		void *priv);
+int target_unregister_event_callback(
+		int (*callback)(struct target *target,
+				enum target_event event, void *priv),
+		void *priv);
+int target_poll(struct target *target);
+int target_resume(struct target *target, int current, uint32_t address,
+		int handle_breakpoints, int debug_execution);
+int target_halt(struct target *target);
+int target_call_event_callbacks(struct target *target, enum target_event event);
 
-/* The period is very approximate, the callback can happen much more often
+/**
+ * The period is very approximate, the callback can happen much more often
  * or much more rarely than specified
  */
-extern int target_register_timer_callback(int (*callback)(void *priv), int time_ms, int periodic, void *priv);
-extern int target_unregister_timer_callback(int (*callback)(void *priv), void *priv);
-extern int target_call_timer_callbacks(void);
-/* invoke this to ensure that e.g. polling timer callbacks happen before
+int target_register_timer_callback(int (*callback)(void *priv),
+		int time_ms, int periodic, void *priv);
+int target_unregister_timer_callback(int (*callback)(void *priv), void *priv);
+
+int target_call_timer_callbacks(void);
+/**
+ * Invoke this to ensure that e.g. polling timer callbacks happen before
  * a syncrhonous command completes.
  */
-extern int target_call_timer_callbacks_now(void);
+int target_call_timer_callbacks_now(void);
 
-extern target_t* get_current_target(struct command_context_s *cmd_ctx);
-extern target_t *get_target(const char *id);
+struct target* get_current_target(struct command_context *cmd_ctx);
+struct target *get_target(const char *id);
 
 /**
- * Get the target name.
+ * Get the target type name.
  *
  * This routine is a wrapper for the target->type->name field.
+ * Note that this is not an instance-specific name for his target.
  */
-extern const char *target_get_name(struct target_s *target);
+const char *target_type_name(struct target *target);
 
 /**
- * Examine the specified @a target.
+ * Examine the specified @a target, letting it perform any
+ * initialization that requires JTAG access.
  *
  * This routine is a wrapper for target->type->examine.
  */
-extern int target_examine_one(struct target_s *target);
-/// @returns @c true if the target has been examined.
-extern bool target_was_examined(struct target_s *target);
-/// Sets the @c examined flag for the given target.
-extern void target_set_examined(struct target_s *target);
-/// Reset the @c examined flag for the given target.
-extern void target_reset_examined(struct target_s *target);
+int target_examine_one(struct target *target);
 
+/// @returns @c true if target_set_examined() has been called.
+static inline bool target_was_examined(struct target *target)
+{
+	return target->examined;
+}
+
+/// Sets the @c examined flag for the given target.
+/// Use in target->type->examine() after one-time setup is done.
+static inline void target_set_examined(struct target *target)
+{
+	target->examined = true;
+}
 
 /**
  * Add the @a breakpoint for @a target.
  *
  * This routine is a wrapper for target->type->add_breakpoint.
  */
-extern int target_add_breakpoint(struct target_s *target,
-		struct breakpoint_s *breakpoint);
+int target_add_breakpoint(struct target *target,
+		struct breakpoint *breakpoint);
 /**
  * Remove the @a breakpoint for @a target.
  *
  * This routine is a wrapper for target->type->remove_breakpoint.
  */
-extern int target_remove_breakpoint(struct target_s *target,
-		struct breakpoint_s *breakpoint);
+int target_remove_breakpoint(struct target *target,
+		struct breakpoint *breakpoint);
 /**
  * Add the @a watchpoint for @a target.
  *
  * This routine is a wrapper for target->type->add_watchpoint.
  */
-extern int target_add_watchpoint(struct target_s *target,
-		struct watchpoint_s *watchpoint);
+int target_add_watchpoint(struct target *target,
+		struct watchpoint *watchpoint);
 /**
  * Remove the @a watchpoint for @a target.
  *
  * This routine is a wrapper for target->type->remove_watchpoint.
  */
-extern int target_remove_watchpoint(struct target_s *target,
-		struct watchpoint_s *watchpoint);
+int target_remove_watchpoint(struct target *target,
+		struct watchpoint *watchpoint);
 
 /**
  * Obtain the registers for GDB.
  *
  * This routine is a wrapper for target->type->get_gdb_reg_list.
  */
-extern int target_get_gdb_reg_list(struct target_s *target,
-		struct reg_s **reg_list[], int *reg_list_size);
+int target_get_gdb_reg_list(struct target *target,
+		struct reg **reg_list[], int *reg_list_size);
 
 /**
  * Step the target.
  *
  * This routine is a wrapper for target->type->step.
  */
-int target_step(struct target_s *target,
+int target_step(struct target *target,
 		int current, uint32_t address, int handle_breakpoints);
 /**
  * Run an algorithm on the @a target given.
  *
  * This routine is a wrapper for target->type->run_algorithm.
  */
-extern int target_run_algorithm(struct target_s *target,
-		int num_mem_params, mem_param_t *mem_params,
-		int num_reg_params, reg_param_t *reg_param,
+int target_run_algorithm(struct target *target,
+		int num_mem_params, struct mem_param *mem_params,
+		int num_reg_params, struct reg_param *reg_param,
 		uint32_t entry_point, uint32_t exit_point,
 		int timeout_ms, void *arch_info);
 
@@ -344,15 +367,26 @@ extern int target_run_algorithm(struct target_s *target,
  *
  * This routine is a wrapper for target->type->read_memory.
  */
-extern int target_read_memory(struct target_s *target,
+int target_read_memory(struct target *target,
 		uint32_t address, uint32_t size, uint32_t count, uint8_t *buffer);
 /**
  * Write @a count items of @a size bytes to the memory of @a target at
- * the @a address given.
+ * the @a address given. @a address must be aligned to @a size
+ * in target memory.
+ *
+ * The endianness is the same in the host and target memory for this
+ * function.
+ *
+ * \todo TODO:
+ * Really @a buffer should have been defined as "const void *" and
+ * @a buffer should have been aligned to @a size in the host memory.
+ *
+ * This is not enforced via e.g. assert's today and e.g. the
+ * target_write_buffer fn breaks this assumption.
  *
  * This routine is wrapper for target->type->write_memory.
  */
-extern int target_write_memory(struct target_s *target,
+int target_write_memory(struct target *target,
 		uint32_t address, uint32_t size, uint32_t count, uint8_t *buffer);
 
 /**
@@ -362,7 +396,7 @@ extern int target_write_memory(struct target_s *target,
  *
  * This routine is wrapper for target->type->bulk_write_memory.
  */
-extern int target_bulk_write_memory(struct target_s *target,
+int target_bulk_write_memory(struct target *target,
 		uint32_t address, uint32_t count, uint8_t *buffer);
 
 /*
@@ -389,14 +423,18 @@ extern int target_bulk_write_memory(struct target_s *target,
  * write operation, thus making this fn suitable to e.g. write to special
  * peripheral registers which do not support byte operations.
  */
-extern int target_write_buffer(struct target_s *target, uint32_t address, uint32_t size, uint8_t *buffer);
-extern int target_read_buffer(struct target_s *target, uint32_t address, uint32_t size, uint8_t *buffer);
-extern int target_checksum_memory(struct target_s *target, uint32_t address, uint32_t size, uint32_t* crc);
-extern int target_blank_check_memory(struct target_s *target, uint32_t address, uint32_t size, uint32_t* blank);
-extern int target_wait_state(target_t *target, enum target_state state, int ms);
+int target_write_buffer(struct target *target,
+		uint32_t address, uint32_t size, uint8_t *buffer);
+int target_read_buffer(struct target *target,
+		uint32_t address, uint32_t size, uint8_t *buffer);
+int target_checksum_memory(struct target *target,
+		uint32_t address, uint32_t size, uint32_t* crc);
+int target_blank_check_memory(struct target *target,
+		uint32_t address, uint32_t size, uint32_t* blank);
+int target_wait_state(struct target *target, enum target_state state, int ms);
 
 /** Return the *name* of this targets current state */
-const char *target_state_name( target_t *target );
+const char *target_state_name( struct target *target );
 
 /* DANGER!!!!!
  *
@@ -409,35 +447,37 @@ const char *target_state_name( target_t *target );
  * upon resuming or resetting the CPU.
  *
  */
-extern int target_alloc_working_area(struct target_s *target, uint32_t size, working_area_t **area);
-extern int target_free_working_area(struct target_s *target, working_area_t *area);
-extern int target_free_working_area_restore(struct target_s *target, working_area_t *area, int restore);
-extern void target_free_all_working_areas(struct target_s *target);
-extern void target_free_all_working_areas_restore(struct target_s *target, int restore);
+int target_alloc_working_area(struct target *target,
+		uint32_t size, struct working_area **area);
+int target_free_working_area(struct target *target, struct working_area *area);
+int target_free_working_area_restore(struct target *target,
+		struct working_area *area, int restore);
+void target_free_all_working_areas(struct target *target);
+void target_free_all_working_areas_restore(struct target *target, int restore);
 
-extern target_t *all_targets;
+extern struct target *all_targets;
 
-extern target_event_callback_t *target_event_callbacks;
-extern target_timer_callback_t *target_timer_callbacks;
+extern struct target_event_callback *target_event_callbacks;
+extern struct target_timer_callback *target_timer_callbacks;
 
-extern uint32_t target_buffer_get_u32(target_t *target, const uint8_t *buffer);
-extern uint16_t target_buffer_get_u16(target_t *target, const uint8_t *buffer);
-extern uint8_t  target_buffer_get_u8 (target_t *target, const uint8_t *buffer);
-extern void target_buffer_set_u32(target_t *target, uint8_t *buffer, uint32_t value);
-extern void target_buffer_set_u16(target_t *target, uint8_t *buffer, uint16_t value);
-extern void target_buffer_set_u8 (target_t *target, uint8_t *buffer, uint8_t  value);
+uint32_t target_buffer_get_u32(struct target *target, const uint8_t *buffer);
+uint16_t target_buffer_get_u16(struct target *target, const uint8_t *buffer);
+uint8_t  target_buffer_get_u8 (struct target *target, const uint8_t *buffer);
+void target_buffer_set_u32(struct target *target, uint8_t *buffer, uint32_t value);
+void target_buffer_set_u16(struct target *target, uint8_t *buffer, uint16_t value);
+void target_buffer_set_u8 (struct target *target, uint8_t *buffer, uint8_t  value);
 
-int target_read_u32(struct target_s *target, uint32_t address, uint32_t *value);
-int target_read_u16(struct target_s *target, uint32_t address, uint16_t *value);
-int target_read_u8(struct target_s *target, uint32_t address, uint8_t *value);
-int target_write_u32(struct target_s *target, uint32_t address, uint32_t value);
-int target_write_u16(struct target_s *target, uint32_t address, uint16_t value);
-int target_write_u8(struct target_s *target, uint32_t address, uint8_t value);
+int target_read_u32(struct target *target, uint32_t address, uint32_t *value);
+int target_read_u16(struct target *target, uint32_t address, uint16_t *value);
+int target_read_u8(struct target *target, uint32_t address, uint8_t *value);
+int target_write_u32(struct target *target, uint32_t address, uint32_t value);
+int target_write_u16(struct target *target, uint32_t address, uint16_t value);
+int target_write_u8(struct target *target, uint32_t address, uint8_t value);
 
 /* Issues USER() statements with target state information */
-int target_arch_state(struct target_s *target);
+int target_arch_state(struct target *target);
 
-void target_handle_event(target_t *t, enum target_event e);
+void target_handle_event(struct target *t, enum target_event e);
 void target_all_handle_event(enum target_event e);
 
 #define ERROR_TARGET_INVALID	(-300)
@@ -452,7 +492,8 @@ void target_all_handle_event(enum target_event e);
 #define ERROR_TARGET_NOT_RUNNING (-310)
 #define ERROR_TARGET_NOT_EXAMINED (-311)
 
-extern const Jim_Nvp nvp_error_target[];
-extern const char *target_strerror_safe(int err);
+const char *target_strerror_safe(int err);
+
+extern bool get_target_reset_nag(void);
 
 #endif /* TARGET_H */
