@@ -26,7 +26,8 @@
 #ifndef TARGET_REQUEST_H
 #define TARGET_REQUEST_H
 
-#include "target.h"
+struct target;
+struct command_context;
 
 typedef enum target_req_cmd
 {
@@ -36,14 +37,15 @@ typedef enum target_req_cmd
 /*	TARGET_REQ_SEMIHOSTING, */
 } target_req_cmd_t;
 
-typedef struct debug_msg_receiver_s
+struct debug_msg_receiver
 {
-	command_context_t *cmd_ctx;
-	struct debug_msg_receiver_s *next;
-} debug_msg_receiver_t;
+	struct command_context *cmd_ctx;
+	struct debug_msg_receiver *next;
+};
 
-extern int target_request(target_t *target, uint32_t request);
-extern int delete_debug_msg_receiver(struct command_context_s *cmd_ctx, target_t *target);
-extern int target_request_register_commands(struct command_context_s *cmd_ctx);
+int target_request(struct target *target, uint32_t request);
+int delete_debug_msg_receiver(struct command_context *cmd_ctx,
+		struct target *target);
+int target_request_register_commands(struct command_context *cmd_ctx);
 
 #endif /* TARGET_REQUEST_H */
