@@ -20,28 +20,33 @@
 #ifndef TRACE_H
 #define TRACE_H
 
-#include "types.h"
+#include <helper/types.h>
 
-struct target_s;
-struct command_context_s;
+struct target;
+struct command_context;
 
-typedef struct trace_point_s
+struct trace_point
 {
 	uint32_t address;
 	uint64_t hit_counter;
-} trace_point_t;
+};
 
-typedef struct trace_s
+struct trace
 {
 	uint32_t num_trace_points;
 	uint32_t trace_points_size;
-	trace_point_t *trace_points;
+	struct trace_point *trace_points;
 	uint32_t trace_history_size;
 	uint32_t *trace_history;
 	uint32_t trace_history_pos;
 	int trace_history_overflowed;
-} trace_t;
+};
 
+/**
+ * \todo This enum is one of the few things in this file related
+ * to *hardware* tracing ... split such "real" tracing out from
+ * the contrib/libdcc support.
+ */
 typedef enum trace_status
 {
 	TRACE_IDLE = 0x0,
@@ -51,8 +56,8 @@ typedef enum trace_status
 	TRACE_OVERFLOWED = 0x8,
 } trace_status_t;
 
-extern int trace_point(struct target_s *target, uint32_t number);
-extern int trace_register_commands(struct command_context_s *cmd_ctx);
+int trace_point(struct target *target, uint32_t number);
+int trace_register_commands(struct command_context *cmd_ctx);
 
 #define ERROR_TRACE_IMAGE_UNAVAILABLE		-(1500)
 #define ERROR_TRACE_INSTRUCTION_UNAVAILABLE	-(1501)
